<?php

class we_base_sessionHandler implements SessionHandlerInterface{
	//prevent crashed or killed sessions to stay
	private $execTime;
	private $sessionName;
	private $DB;
	private $id = 0;
	private $crypt = false;
	private $hash = '';
	public static $releaseError = false;
	public static $acquireLock = 0;

//	private static $debug = false;

	public function __construct(){
		if(session_status() === PHP_SESSION_ACTIVE){
			return;
		}
		/* if(!empty($_REQUEST['debug'])){
		  self::$debug = true;
		  } */
		//setup session vars
		ini_set('session.hash_function', 1); //set sha-1 which will generate 40 bytes of session_id
		ini_set('session.hash_bits_per_character', 4);
		//php 7.1
		ini_set('session.sid_bits_per_character', 4);
		ini_set('session.sid_length', 40);
		if(!defined('SYSTEM_WE_SESSION') || !SYSTEM_WE_SESSION || $this->id){
			session_start();
			return;
		}
		ini_set('session.gc_probability', 1);
		ini_set('session.use_strict_mode', 1);
		ini_set('session.gc_divisor', 100);
		session_set_save_handler($this, true);
		$this->DB = new DB_WE();
		$this->execTime = max(min(60, intval(get_cfg_var('max_execution_time'))), 5); //time might be wrong (1&1); make exectime at least 5 seconds which is quite small
		$this->id = md5(uniqid('', true));
		if(!(extension_loaded('suhosin') && ini_get('suhosin.session.encrypt')) && defined('SYSTEM_WE_SESSION_CRYPT') && SYSTEM_WE_SESSION_CRYPT){
			$key = $_SERVER['DOCUMENT_ROOT'] . (!empty($_SERVER['HTTP_USER_AGENT']) ? $_SERVER['HTTP_USER_AGENT'] : 'HTTP_USER_AGENT');
			if(SYSTEM_WE_SESSION_CRYPT === 2){
				if(!isset($_COOKIE['secure'])){
					$cookie = session_get_cookie_params();
					$_COOKIE['secure'] = we_users_user::getHashIV(30);
					setcookie('secure', $_COOKIE['secure'], 0, '/', $cookie['domain'], $cookie['secure'], true);
				}
				$key .= $_COOKIE['secure'];
			}
			// due to IE we can't use HTTP_ACCEPT_LANGUAGE, HTTP_ACCEPT_ENCODING - they change the string on each request
			$this->crypt = hash('haval224,4', $key);
			//double key size is needed
			$this->crypt .= $this->crypt;
		}
		session_start();
	}

	public function __destruct(){
		if(defined('SYSTEM_WE_SESSION') && SYSTEM_WE_SESSION && session_status() === PHP_SESSION_ACTIVE){
			session_write_close();
		}
	}

	public function open($savePath, $sessName){
		$this->sessionName = $sessName;
		return true;
	}

	public function close(){//FIX for php >5.5, where write is only called, if sth. in session changed
		/* if(self::$debug){
		  t_e(session_id());
		  } */
		return true;
	}

	public function read($sessID){
		//$sessID = $this->DB->escape(self::getSessionID($sessID));
		$lock = microtime(true);
		if(f('SELECT 1 FROM ' . SESSION_TABLE . ' WHERE touch>=NOW()-INTERVAL ' . SYSTEM_WE_SESSION_TIME . ' second AND session_id=x\'' . $sessID . '\' AND sessionName="' . $this->sessionName . '"', '', $this->DB)){//session exists
			$max = $this->execTime * 6; //just wait 80% of the time to clear the lock, if not cleared, break the lock

			while(!($this->DB->query('UPDATE ' . SESSION_TABLE . ' SET
	lock_id=x\'' . $this->id . '\',
	lockTime=NOW()
WHERE
	session_id=x\'' . $sessID . '\' AND
	sessionName="' . $this->sessionName . '"' . ( --$max ? ' AND
	(lock_id IS NULL OR lock_id=x\'' . $this->id . '\' OR lockTime+INTERVAL ' . $this->execTime . ' second<NOW())' : '')
			) &&
			$this->DB->affected_rows())){
				self::$releaseError = self::$releaseError ?: getHash('SELECT sessionName,HEX(session_id) AS session_id,lockTime,HEX(lock_id) AS lock_id,touch,NOW()'/* ,debug */ . ' FROM ' . SESSION_TABLE . ' WHERE session_id=x\'' . $sessID . '\' AND sessionName="' . $this->sessionName . '"');
				usleep(250000);
			}
			//now we have a session lock, read data
			$data = f('SELECT session_data FROM ' . SESSION_TABLE . ' WHERE	session_id=x\'' . $sessID . '\' AND	sessionName="' . $this->sessionName . '"');

			self::$acquireLock = microtime(true) - $lock;
			if(!$max){
				self::$releaseError['myLock'] = $this->id;
			} else {
				self::$releaseError = '';
			}

			if($data){
				$data = ($data[0] === '$' && $this->crypt ? we_customer_customer::decryptData($data, $this->crypt) : $data);
				if($data && $data[0] === 'x'){
					//valid gzip
					$data = gzuncompress($data);
					$this->hash = md5($sessID . $data);
					return $data;
				}//else we need a new sessionid; if decrypt failed we might else destroy an existing valid session
			}
		}
		self::$acquireLock = microtime(true) - $lock;
		//if we don't find valid data, generate a new ID because of session stealing
		$this->write($sessID, '', true);
		return '';
	}

	public function write($sessID, $sessData, $lock = false){
		if(!$sessData && !$lock){
			return $this->destroy($sessID);
		}
		//$sessID = self::getSessionID($sessID);
		if(md5($sessID . $sessData) == $this->hash){//if nothing changed,we don't have to bother the db
			$this->DB->query('UPDATE ' . SESSION_TABLE . ' SET ' .
				we_database_base::arraySetter([
					'lock_id' => sql_function($lock ? 'x\'' . $this->id . '\'' : 'NULL'),
					'lockTime' => sql_function($lock ? 'NOW()' : 'NULL'),
				]) . ' WHERE session_id=x\'' . $sessID . '\' AND sessionName="' . $this->sessionName . '"');

			if($this->DB->affected_rows()){//make sure we had an successfull update
				return true;
			}
		}

		$sessData = SYSTEM_WE_SESSION_CRYPT && $this->crypt ? we_customer_customer::cryptData(gzcompress($sessData, 4), $this->crypt, true) : gzcompress($sessData, 4);

		$this->DB->query('REPLACE INTO ' . SESSION_TABLE . ' SET ' . we_database_base::arraySetter([
				'sessionName' => $this->sessionName,
				'session_id' => sql_function('x\'' . $sessID . '\''),
				'session_data' => sql_function('x\'' . bin2hex($sessData) . '\''),
				'lock_id' => sql_function($lock ? 'x\'' . $this->id . '\'' : 'NULL'),
				'lockTime' => sql_function($lock ? 'NOW()' : 'NULL'),
				/* 'debug' => print_r([
				  'URI' => $_SERVER['REQUEST_URI'],
				  'oldLock'=>bin2hex($this->id),
				  'Cookie' => $_COOKIE,
				  'Req' => $_REQUEST,
				  ], true) */
		]));
		return true;
	}

	public function destroy($sessID){
		unset($_SESSION);
		self::delSess($sessID, $this->sessionName);
		return true;
	}

	private static function delSess($sessID, $sessName){
		//$sessID = self::getSessionID($sessID);
		$GLOBALS['DB_WE']->query('DELETE FROM ' . SESSION_TABLE . ' WHERE session_id=x\'' . $sessID . '\' AND sessionName="' . $sessName . '"');
	}

	public function gc($sessMaxLifeTime){
		self::cleanSessions($this->DB);
		return true;
	}

	public static function cleanSessions(we_database_base $db){
		$db->query('DELETE FROM ' . SESSION_TABLE . ' WHERE touch<NOW()-INTERVAL ' . SYSTEM_WE_SESSION_TIME . ' second');
		$db->query('OPTIMIZE TABLE ' . SESSION_TABLE);
	}

	private static function session_create_id(){
		session_regenerate_id();
		return session_id();
	}

	static function makeNewID($destroy = false){
		if($destroy){
			session_destroy();
			session_start();
		} else {
			//we need a new lock on the generated id, since partial data is sent to the browser, subsequent calls with the new sessionid might happen
			if(SYSTEM_WE_SESSION){
				self::delSess(session_id(), session_name());
				self::session_create_id();
			} else {
				session_regenerate_id(true);
			}

			session_write_close();
			session_start();
		}
	}

	public static function getCurrentHex(){
		$sessID = session_id();
		return preg_match('|^([a-f0-9]){32,40}$|', $sessID) ? $sessID : bin2hex($sessID);
	}

	public function updateTimestamp($sessID, $val){
		$this->DB->query('UPDATE ' . SESSION_TABLE . ' SET lock_id=NULL,lockTime=NULL WHERE session_id=x\'' . $sessID . '\' AND sessionName="' . $this->sessionName . '" AND lock_id=x\'' . $this->id . '\'');
		return true;
	}

	public function validateId($sessID){
		return preg_match('|^([a-f0-9]){32,40}$|', $sessID) === 1;
	}

}
