<?php

/**
 * webEdition CMS
 *
 * $Rev: 096b1a1b079c $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-10-08 17:38 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
/* the parent class of storagable webEdition classes */
class we_category_view extends we_modules_view{

	function __construct(){
		parent::__construct();
		$this->Model = new we_category_category();
	}

	function getJSTop(){
		$modData = we_modules_info::getModuleField(we_modules_info::CATEGORY, 'text');
		$title = ($modData ? 'webEdition ' . g_l('global', '[modules]') . ' - ' . $modData : '');

		return
				parent::getJSTop() .
				we_html_element::jsScript(WE_JS_MODULES_DIR . '/category/category_top.js', "parent.document.title='" . $title . "'");
	}

	function getJSProperty(array $jsVars = []){
		return we_html_element::jsScript(WE_JS_MODULES_DIR . '/category/category_prop.js');
	}

	private function renameChildrenPath($id){
		$db = $this->db;
		$parentPath = f('SELECT Path FROM ' . CATEGORY_TABLE . ' WHERE ID=' . intval($id));
		$db->query('UPDATE ' . CATEGORY_TABLE . ' SET Path=CONCAT("' . $db->escape($parentPath) . '","/",Text) WHERE ParentID=' . intval($id));
		$updates = $db->getAllq('SELECT ID FROM ' . CATEGORY_TABLE . ' WHERE ParentID=' . intval($id), true);
		foreach($updates as $id){
			$this->renameChildrenPath($id);
		}
	}

	private function CatInUse($id){
		$childs = $this->getChilds($id);
		$childs[] = $id;
		if(f('SELECT 1 FROM ' . FILE_TABLE . ' f JOIN ' . CATEGORY_TABLE . ' c ON (FIND_IN_SET(c.ID,f.Category) OR FIND_IN_SET(c.ID,f.temp_category) ) WHERE c.ID IN (' . implode(',', $childs) . ') LIMIT 1', '', $this->db) ||
				(we_modules_info::isActive(we_modules_info::OBJECT) && f('SELECT 1 FROM ' . OBJECT_FILES_TABLE . ' `of` JOIN ' . CATEGORY_TABLE . ' c ON (FIND_IN_SET(c.ID,`of`.Category)) WHERE c.ID IN (' . implode(',', $childs) . ') LIMIT 1', '', $this->db))){
			return true;
		}
		return false;
	}

	private function getChilds($id){
		$ret = [];
		$ids = [$id];
		while(($ids = $this->db->getAllq('SELECT ID FROM ' . CATEGORY_TABLE . ' WHERE ParentID IN (' . implode(',', $ids) . ')', true))){
			$ret = array_merge($ret, $ids);
		}
		return $ret;
	}

	private function delDir($id){
		if(!$id){
			return;
		}
		$entries = $this->db->getAllq('SELECT ID FROM ' . CATEGORY_TABLE . ' WHERE ParentID=' . intval($id), true);
		foreach($entries as $entry){
			$this->delDir($entry);
		}
		$entries = $this->db->getAllq('SELECT ID FROM ' . CATEGORY_TABLE . ' WHERE ParentID=' . intval($id), true);
		$entries[] = $id;
		$this->delEntries($entries);
	}

	private function delEntries(array $ids){
		$this->db->query('DELETE FROM ' . CATEGORY_TABLE . ' WHERE ID IN (' . implode(',', $ids) . ')');
	}

	function processCommands(we_base_jsCmd $jsCmd){
		switch(we_base_request::_(we_base_request::STRING, 'we_cmd', '', 0)){
			case 'we_save':
				if(!we_base_permission::hasPerm(we_base_perm::WORK_CATEGORY_EDIT)){
					$jsCmd->addMsg(g_l('weClass', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					break;
				}

				$this->Model->isnew = empty($this->Model->ID); // FIXME: if init was ok we wouldn't need this!

				if(empty($this->Model->Text)){
					$jsCmd->addMsg(g_l('weEditor', '[category][filename_empty]'), we_base_util::WE_MESSAGE_ERROR);
				} else if(strpos(',', $this->Model->Text) !== false){
					$jsCmd->addMsg(g_l('weEditor', '[category][name_komma]'), we_base_util::WE_MESSAGE_ERROR);
				} elseif(f('SELECT 1 FROM ' . CATEGORY_TABLE . ' WHERE ID!=' . intval($this->Model->ID) . ' AND ParentID=' . $this->Model->ParentID . ' AND Text="' . $this->db->escape($this->Model->Text) . '" LIMIT 1', '', $this->db)){
					$jsCmd->addMsg(sprintf(g_l('weEditor', '[category][response_path_exists]'), $this->Model->Path), we_base_util::WE_MESSAGE_ERROR);
				} elseif(preg_match('|[\\\'"<>/]|', $this->Model->Text)){
					$jsCmd->addMsg(sprintf(g_l('weEditor', '[category][we_filename_notValid]'), $this->Model->Path), we_base_util::WE_MESSAGE_ERROR);
				} elseif($this->Model->save()){
					$jsCmd->addMsg(sprintf(g_l('thumbnails', '[saved_successfully]'), $this->Model->Text), we_base_util::WE_MESSAGE_NOTICE);
					if($this->Model->wasUpdate){
						$this->renameChildrenPath($this->Model->ID);
						$jsCmd->addCmd('updateTreeEntry', ['id' => $this->Model->ID, 'parentid' => $this->Model->ParentID, 'text' => $this->Model->Text]);
					} else {
						//update parent
						if($this->Model->ParentID){
							$jsCmd->addCmd('updateTreeEntry', ['id' => $this->Model->ParentID, 'typ' => we_tree_base::TYP_FOLDER, 'contenttype' => we_base_ContentTypes::CATEGORIES]);
						}
						$jsCmd->addCmd('makeTreeEntry', ['id' => $this->Model->ID, 'parentid' => $this->Model->ParentID, 'text' => $this->Model->Text, 'open' => false, 'contenttype' => we_base_ContentTypes::CATEGORY,
							'table' => CATEGORY_TABLE, 'published' => 1]);
					}
				} else {
					echo "ERROR";
				}
				break;
			case 'delete_category':
				if(!$this->Model->ID || !we_base_permission::hasPerm(we_base_perm::WORK_CATEGORY_EDIT)){
					$jsCmd->addMsg(g_l('weClass', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					return;
				}
				$catsToDel = [$this->Model->ID];
				$finalDelete = [];
				$catlistNotDeleted = '';
				foreach($catsToDel as $id){
					if($this->CatInUse($id)){
						$catlistNotDeleted .= id_to_path($id, CATEGORY_TABLE, $this->db) . "\n";
					} else {
						$finalDelete[] = $id;
					}
				}
				if($finalDelete){
					foreach($finalDelete as $id){
						$this->delDir($id);
						$jsCmd->addCmd('deleteTreeEntry', $id);

						if($id === $this->Model->ID){ // currently opened cat is deleted: replace editor by home
							$jsCmd->addCmd('unsetHot');
							$jsCmd->addCmd('loadHome');
							$_SESSION['weS']['category_session'] = [];
							$this->Model = new we_category_category();
						}
					}
				}
				if($catlistNotDeleted){
					$jsCmd->addMsg(g_l('fileselector', '[cat_in_use]') . "\n\n" . $catlistNotDeleted, we_base_util::WE_MESSAGE_ERROR);
				}
				break;
			default:
				$id = (we_base_request::_(we_base_request::INT, 'we_cmd', 0, 1) ?: 0);
				if($id){
					$this->Model->load($id);
					break;
				}
				$this->Model = new we_category_category();
		}
		$this->Model->saveInSession($_SESSION['weS']['category_session']);
	}

	function processVariables(){
		if(!empty($_SESSION['weS']['category_session'])){
			$this->Model->we_initSessDat($_SESSION['weS']['category_session']);

			$this->Model->Text = we_base_request::_(we_base_request::STRING, 'Text');
			$this->Model->ParentID = we_base_request::_(we_base_request::INT, 'FolderID');
			$this->Model->Path = ($this->Model->ParentID ? f('SELECT Path FROM ' . CATEGORY_TABLE . ' WHERE ID=' . intval($this->Model->ParentID)) : '') . '/' . $this->Model->Text;

			$this->Model->Title = we_base_request::_(we_base_request::STRING, 'catTitle');
			$this->Model->Description = we_base_request::_(we_base_request::RAW_CHECKED, 'catDescription'); //keep HTML Code
			//$this->Model->isnew = false; // FIXME: make isnew persistent or
		} else {
			$_SESSION['weS']['category_session'] = [];
		}
	}

	public function formName(){
		$path = f('SELECT cc.Path FROM ' . CATEGORY_TABLE . ' c LEFT JOIN ' . CATEGORY_TABLE . ' cc ON c.ParentID=cc.ID WHERE c.ID=' . intval($this->Model->ID)) ?: '/';

		$weSuggest = &we_gui_suggest::getInstance();
		$weSuggest->setAcId('Doc');
		$weSuggest->setTable(CATEGORY_TABLE);
		$weSuggest->setContentType('');
		$weSuggest->setInput('FolderIDPath', $path);
		$weSuggest->setMaxResults(20);
		$weSuggest->setRequired(true);
		$weSuggest->setResult('FolderID', $this->Model->ParentID);
		$weSuggest->setSelector(we_gui_suggest::DirSelector);
		$weSuggest->setWidth(250);
		$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_file', document.we_form.elements.FolderID.value, WE().consts.tables.CATEGORY_TABLE, 'FolderID', 'FolderIDPath')"));

		return we_html_tools::formElementTable(g_l('weClass', '[category]'), we_html_tools::textInput('Text', $this->Model->Text, '', ['id' => 'category'], 'text', 360)) .
				we_html_tools::formElementTable(g_l('weClass', '[dir]'), $weSuggest->getHTML());
	}

	public function formSettings(){
		return we_html_tools::formElementTable(g_l('global', '[title]'), we_html_tools::textInput('catTitle', $this->Model->Title, '', [], 'text', 360)) .
				we_html_tools::formElementTable(g_l('global', '[description]'), we_html_forms::weTextarea('catDescription', $this->Model->Description, [
							'bgcolor' => 'white',
							'inlineedit' => 'true',
							'wysiwyg' => 'true',
							'width' => 450,
							'height' => 130,
							'commands' => 'prop,fontsize,xhtmlxtras,color,justify,list,link,table,insert,fullscreen,visibleborders,editsource'
								], false, '', true, true, true, true, $GLOBALS['we']['Charset']['Backend']));
	}

	public function getHomeScreen(){
		return parent::getActualHomeScreen(we_modules_info::CATEGORY, we_html_button::create_button('fat:add,fa-lg fa-plus', "javascript:we_cmd('add_category')"), we_html_element::form([
							'name' => 'we_form'], $this->getCommonHiddens(['cmd' => 'home', 'pnt' => 'edbody']) . we_html_element::hidden('home', 0)));
	}

}
