<?php

/**
 * webEdition CMS
 *
 * $Rev: 692bfbe32361 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2018-10-07 22:12 +0000 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_exim_contentProvider{
	const CODING_NONE = '';
	const CODING_ENCODE = 'encode';
	const CODING_SERIALIZE = 'serial';
	const CODING_ATTRIBUTE = 'coding';
	const DONT_IMEXPORT_PROPS = ['elements', 'usedElementNames', 'wasUpdate', 'nHash'];

	static function getInstance($we_ContentType, $ID = '', $table = ''){
		$we_doc = '';
		if($ID){
			//used in include!
			$we_ID = $ID;
		}
		switch($we_ContentType){
			case 'doctype':
				$we_doc = new we_doctype_docTypes();
				if($ID){
					$we_doc->initByID($ID, $we_doc->Table);
				}
				return $we_doc;
			case 'category':
				$we_doc = new we_category_category();
				$we_doc->load($ID);
				return $we_doc;
			case we_base_ContentTypes::NAVIGATION:
				$we_doc = new we_navigation_navigation();
				$we_doc->we_load($ID);
				return $we_doc;
			case we_base_ContentTypes::NAVIGATIONRULE:
				$we_doc = new we_navigation_rule();
				$we_doc->we_load($ID);
				return $we_doc;
			case we_base_ContentTypes::THUMBNAIL:
				$we_doc = new we_exim_thumbnailExport();
				$we_doc->we_load($ID);
				return $we_doc;
			case 'we_backup_tableAdv':
			case 'we_backup_table':
				$we_doc = new we_backup_tableAdv($table);
				return $we_doc;
			case 'we_backup_tableItem':
				$we_doc = new we_backup_tableItem($table);
				if($ID){
					$we_doc->load($ID);
				}
				return $we_doc;

			case 'we_backup_binary':
			case 'weBinary':
				$we_doc = new we_backup_binary();
				$we_doc->load($ID, false);
				return $we_doc;
			case 'weVersion':
				$we_doc = new we_backup_version();
				$we_doc->load($ID, false);
				return $we_doc;
			// fix for classes
			case 'object':
				$we_doc = new we_backup_object();
				$we_doc->initByID($ID, OBJECT_TABLE);
				return $we_doc;
			// fix ends ------------------------------------------------
		}

		switch($we_ContentType){
			case we_base_ContentTypes::FOLDER:
				$we_Table = $table ?: FILE_TABLE;
				break;
			case we_base_ContentTypes::TEMPLATE:
				$we_Table = TEMPLATES_TABLE;
				break;
			case we_base_ContentTypes::OBJECT:
				$we_Table = OBJECT_TABLE;
				break;
			case we_base_ContentTypes::OBJECT_FILE:
				$we_Table = OBJECT_FILES_TABLE;
				break;
			default:
				$we_Table = FILE_TABLE;
		}
		return we_contents_document::initDoc([], $we_ContentType, $we_ID, $we_Table, true);
	}

	static function getTagName($object){
		switch(get_class($object)){
			case 'we_template':
				return 'we:template';
			case 'we_exim_element':
			case 'we_element':
				return 'we:content';
			case 'we_object':
				return 'we:class';
			case 'we_objectFile':
				return 'we:object';
			case 'we_docTypes':
			case 'we_doctype_docTypes':
				return 'we:doctype';
			case 'we_category_category':
			case 'we_category':
				return 'we:category';

			case 'we_backup_tableAdv'://for backup
				return 'we:tableadv';
			case 'we_backup_tableItem'://for backup
				return 'we:tableitem';
			case 'we_backup_binary':
				return 'we:binary';
			case 'we_navigation_navigation':
				return 'we:navigation';
			case 'we_navigation_rule':
				return 'we:navigationrule';
			case 'we_thumbnail':
			case 'we_contents_thumbnail':
			case 'we_thumbnailEx':
				return 'we:thumbnail';
			default:
				return 'we:document';
		}
	}

	static function needCoding($prop, $data){
		if($prop === 'schedArr'){
			return true;
		}
		return preg_match('!(^[asO]:\d+:)|(^\[)|(^\{)|([\x0-\x08\x0e-\x19\x11\x12<>&\x7F-\xFF])!', $data); //exclude x9:\t,x10:\n,x13:\r,x20:space
	}

	static function needCdata($content){
		return preg_match('-[<>&]-', $content);
	}

	static function needSerialize(&$object, $classname, $prop){
		if(!isset($object->$prop)){
			return false;
		}
		if(is_array($object->$prop)){
			return true;
		}
		/* $serialize = ['we_object' => ['SerializedArray'],
		  'we_objectFile' => ['DefArray', 'schedArr']
		  ];

		  if($prop === 'Dat' && $classname === 'we_element' && $object->Name == we_base_variants::ELEMENT_NAME){
		  // exception for shop - handling arrays in the content
		  return true;
		  }
		  if(isset($serialize[$classname])){
		  return in_array($prop, $serialize[$classname]);
		  } */
		return false;
	}

	static function binary2file(&$object, $file, $fwrite = 'fwrite', $tag = 'we:binary'){
		$attribs = '';
		foreach($object->persistent_slots as $v){
			switch($v){
				case 'Data':
				case 'SeqN':
					break;
				default:
					if(isset($object->$v)){
						$content = $object->$v;
					}
					if(self::needCoding($v, $content) || self::needCdata($content) || self::needSerialize($object->ClassName, $v, $content)){//fix for faulty parser
						$content = self::encode($content);
						$coding = [self::CODING_ATTRIBUTE => self::CODING_ENCODE];
					} else {
						$coding = [];
					}
					$attribs .= we_xml_composer::we_xmlElement($v, $content, $coding);
			}
		}

		if(isset($object->Path)){
			$offset = 0;
			$rsize = 1048576;
			do{
				//prefer doc_root over site.
				//FIXME: this must be changed, if parking of documents is implemented
				$path = $_SERVER['DOCUMENT_ROOT'] . $object->Path;
				if($object->Path == ''){
					break;
				}
				if(!file_exists($path)){
					$path = $_SERVER['DOCUMENT_ROOT'] . SITE_DIR . $object->Path;
				}
				$data = we_base_file::loadPart($path, $offset, $rsize);
				if($data){
					$fwrite($file, '<' . $tag . '>' . $attribs .
						we_xml_composer::we_xmlElement('SeqN', $object->SeqN) .
						we_xml_composer::we_xmlElement('Data', self::encode($data), [self::CODING_ATTRIBUTE => self::CODING_ENCODE]) .
						'</' . $tag . '>' . we_backup_util::backupMarker . "\n");
					$offset += $rsize;
					$object->SeqN++;
				}
				// if offset g.t. filesize then exit
				/* if(filesize($path)<$offset){
				  $data = null;
				  } */
			} while($data);
		}
	}

	static function version2file(&$object, $file, $fwrite = 'fwrite'){
		self::binary2file($object, $file, $fwrite, 'we:version');
	}

	static function object2xml($object, $file, array $attribs = [], $fwrite = 'fwrite'){
		$classname = get_class($object);

		switch($classname){
			case 'we_navigation_navigation':
			case 'weNavigation':
				$object->persistent_slots['ClassName'] = we_base_request::STRING;
				break;
			case 'we_navigation_rule':
			case 'weNavigationRule':
			case 'we_category_category':
			case 'we_category':
			case 'we_thumbnailEx':
			case 'we_thumbnail':
			case 'we_contents_thumbnail':
				$object->persistent_slots = array_merge(['ClassName'], $object->persistent_slots);
				break;
			default:
				break;
		}

		$tagName = self::getTagName($object);

		//write tag name
		$write = '<' . $tagName . ($attribs ? we_xml_composer::buildAttributesFromArray($attribs) : '') . '>';

		// fix for classes; insert missing field length into default values ---
		switch($classname){
			case 'we_object':
				$tableInfo = $GLOBALS['DB_WE']->metadata(OBJECT_X_TABLE . $object->ID);
				$defvalues = we_unserialize($object->DefaultValues);
				foreach($tableInfo as $cur){
					$fieldname = $cur['name'];
					if(isset($defvalues[$fieldname])){
						$defvalues[$fieldname]['length'] = ($cur['len'] > 255) ? 255 : $cur['len'];
					}
				}
				$object->DefaultValues = we_serialize($defvalues, SERIALIZE_JSON);
				break;
			// fix ends -----------------------------------------------------------

			case 'we_webEditionDocument':
				$object->TemplatePath = we_base_file::clearPath('/' . str_replace($_SERVER['DOCUMENT_ROOT'], '', $object->TemplatePath));
				$object->DocTypeName = f('SELECT DocType FROM ' . DOC_TYPES_TABLE . ' dt WHERE dt.ID=' . intval($object->DocType));
				$object->persistent_slots[] = 'DocTypeName';
				break;
		}

		if(isset($object->Table)){
			$object->Table = strtolower(stripTblPrefix($object->Table));
		}


		foreach($object->persistent_slots as $k => $v){
			if(!is_numeric($k)){
				//new persistents have the form name=>type
				$v = $k;
			}
			if(in_array($v, self::DONT_IMEXPORT_PROPS)){
				continue;
			}
			if(self::needSerialize($object, $classname, $v)){
				$content = self::encode(we_serialize($content, SERIALIZE_JSON));
				$coding = [self::CODING_ATTRIBUTE => self::CODING_SERIALIZE];
			} else {
				$content = (isset($object->$v) ? $object->$v : '');
				if(self::needCoding($v, $content) || self::needCdata($content)){//fix for faulty parser
					$content = self::encode($content);
					$coding = [self::CODING_ATTRIBUTE => self::CODING_ENCODE];
				} else {
					$coding = [];
				}
			}
			$write .= we_xml_composer::we_xmlElement($v, $content, $coding);
		}
		$fwrite($file, $write);

		if(isset($object->elements) && $object->ClassName != 'we_object'){
			$elements_ids = array_keys($object->elements);

			foreach($elements_ids as $ck){
				$refObj = $object->elements[$ck];
				switch($object->ClassName){
					case 'we_backup_tableAdv':
						$contentObj = new we_exim_element($refObj);
						foreach($refObj as $okey => $ov){
							$contentObj->$okey = trim($ov);
						}
						break;

					default:
						$options = [
							'Name' => $ck,
						];

						if(isset($refObj['dat'])){
							$options['Dat'] = $refObj['dat'];
						}

						if(isset($refObj['type'])){
							$options['Type'] = $refObj['type'];
						}
						if(isset($refObj['len'])){
							$options['Len'] = $refObj['len'];
						}
						if(isset($refObj[we_contents_base::ETYPE_NUMBER])){
							$options['BDID'] = $refObj[we_contents_base::ETYPE_NUMBER];
						}

						$contentObj = new we_exim_element($options);
				}

				self::object2xml($contentObj, $file);
			}
			unset($elements_ids, $contentObj);
			//$out.=$elements_out;
		}

		//return $out;
		$fwrite($file, '</' . $tagName . '>');
	}

	static function file2xml($file, $fh){//FIXME: unused?
		$bin = self::getInstance('weBinary', 0);
		$bin->Path = $file;

		self::binary2file($bin, $fh, false);
	}

	static function isBinary($id){
		return f('SELECT 1 FROM ' . FILE_TABLE . ' WHERE ID=' . intval($id) . ' AND ContentType="' . we_base_ContentTypes::IMAGE . '" OR ContentType LIKE "application/%"  LIMIT 1') == 1;
	}

	static function getCDATA($data){
		return sprintf('<![CDATA[%s]]>', $data);
	}

	static function encode($data){
		return base64_encode($data);
	}

	static function decode($data){
		return base64_decode($data);
	}

	static function getContentTypeHandler($contenttype){
		switch($contenttype){
			case 'category':
				return 'we_base_model';
			case we_base_ContentTypes::TEMPLATE:
				return 'we_template';
			case 'doctype':
				return 'we_doctype_docTypes';
			default:
				return $contenttype;
		}
	}

	public static function getDecodedData($type, $data, $key = ''){
		switch($type){
			case self::CODING_ENCODE:
				return self::decode($data);
			case self::CODING_SERIALIZE:
				$dec = self::decode($data);
				return we_unserialize($dec, '', true) ?: $dec;
			case self::CODING_NONE:
			default:
				return $data;
		}
	}

}
