<?php

/**
 * webEdition CMS
 *
 * $Rev: d40e088df6a7 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2020-02-07 12:00 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_fileupload_resp_import extends we_fileupload_resp_base{

	public function __construct($name = ''){
		parent::__construct($name);
		$this->importFile = true;
	}

	protected function initByHttp(){
		parent::initByHttp();
		if(we_modules_info::isActive(we_modules_info::CATEGORY)){
		if(($catarray = we_base_request::_(we_base_request::STRING_LIST, 'fu_doc_categories', false))){
			$cats = [];
			foreach($catarray as $cat){
				// bugfix Workarround #700
				$cats[] = (is_numeric($cat) ? $cat : path_to_id($cat, CATEGORY_TABLE, $GLOBALS['DB_WE']));
			}
			$categories = implode(',', $cats);
		} else {
			$categories = we_base_request::_(we_base_request::INTLIST, 'fu_doc_categories', we_base_request::NOT_VALID);
		}
		}else{
			$categories = '';
		}
		// FIXME: beim überschreiben bestehender grafiken mit einem neuen bild bleiben width und height in der db unveraendert, so dass beim bearbeiten verzerrt wird
		// FIXME: eliminate all serverside editing opts!
		$this->docVars = array_filter([
			'transaction' => we_base_request::_(we_base_request::TRANSACTION, 'we_transaction', $this->docVars['transaction']),
			'importMetadata' => we_base_request::_(we_base_request::BOOL, 'fu_doc_importMetadata', $this->docVars['importMetadata']),
			'isSearchable' => we_base_request::_(we_base_request::BOOL, 'fu_doc_isSearchable', $this->docVars['isSearchable']),
			'focusX' => we_base_request::_(we_base_request::FLOAT, 'fu_doc_focusX', $this->docVars['focusX']),
			'focusY' => we_base_request::_(we_base_request::FLOAT, 'fu_doc_focusY', $this->docVars['focusY']),
			// unset the following entries when not in request!
			'categories' => $categories,
			'title' => we_base_request::_(we_base_request::STRING, 'fu_doc_title', we_base_request::NOT_VALID),
			'alt' => we_base_request::_(we_base_request::STRING, 'fu_doc_alt', we_base_request::NOT_VALID),
			'thumbs' => we_base_request::_(we_base_request::INTLIST, 'fu_doc_thumbs', we_base_request::NOT_VALID),
			], function($var){
			return $var !== we_base_request::NOT_VALID;
		}
		);

		$this->setImageEditProps();
		$this->saveImageEditPropsInSession();
	}

	public function setImageEditProps($props = []){
		$this->imageEditProps = array_merge($this->imageEditProps, [
			'parentID' => $this->fileVars['parentID'],
			'sameName' => $this->fileVars['sameName'],
			'importMetadata' => $this->docVars['importMetadata'],
			'isSearchable' => $this->docVars['isSearchable'],
			'thumbnails' => empty($this->docVars['thumbs']) ? '' : $this->docVars['thumbs'],
			'categories' => empty($this->docVars['categories']) ?: $this->docVars['categories']
		]);
	}

	protected function postprocess(){
		$we_doc = $this->getWebeditionDocument();

		//TODO: make concise mime and extension test here, taking uploader's typecondition from session...
		if(is_array($we_doc) && isset($we_doc['error'])){
			$response = $we_doc; // this is absurde...
		} else {
			$response = $this->writeWebeditionDocument($we_doc);
		}
		$messageSingleFile = isset($response['weDoc']['text']) ? '</br>- ' . $response['weDoc']['text'] : '';

		return ($response['success'] ?
			array_merge($this->response, ['messageSingleFile' => $messageSingleFile, 'status' => 'success', 'completed' => 1, 'weDoc' => $response['weDoc']]) :
			array_merge($this->response, ['status' => 'failure', 'message' => $response['error']])
			);
	}

	protected function isParentIdOk($parentID = 0){
		return ($ws = get_ws(FILE_TABLE, true)) ? (we_users_util::in_workspace($parentID, $ws, FILE_TABLE)) : true;
	}

	protected function getWebeditionDocument(){ // TODO: avoid some more redundancy in this fn
		if($this->docVars['transaction']){ // import new binary for existing wedoc
			if(!isset($_SESSION['weS']['we_data'][$this->docVars['transaction']])){
				return ['error' => 'transaction is not correct',
					'success' => false,
					'weDoc' => ['id' => 0, 'path' => '']
				];
			}

			$we_dt = $_SESSION['weS']['we_data'][$this->docVars['transaction']];
			$we_doc = we_contents_document::initDoc($we_dt);
			if(!$this->isParentIdOk($we_doc->ParentID)){
				return ['error' => 'workspace not ok',
					'success' => false,
					'weDoc' => ['id' => 0, 'path' => '']
				];
			}

			$we_doc->Extension = strtolower((strpos($this->fileVars['weFileName'], '.') > 0) ? preg_replace('/^.+(\..+)$/', '${1}', $this->fileVars['weFileName']) : ''); //strtolower for feature 3764

			if((!$we_doc->Filename) || (!$we_doc->ID)){ // new document not yet saved
				// Bug Fix #6284
				$we_doc->Filename = preg_replace('/[^A-Za-z0-9._-]/', '', $this->fileVars['weFileName']);
				$we_doc->Filename = preg_replace('/^(.+)\..+$/', '${1}', $we_doc->Filename);
				$we_doc->Text = $we_doc->Filename . $we_doc->Extension;
				$we_doc->Path = $we_doc->getParentPath() . (($we_doc->getParentPath() != '/') ? '/' : '') . $we_doc->Text;

				if(f('SELECT ID FROM ' . FILE_TABLE . ' WHERE Path="' . $GLOBALS['DB_WE']->escape($we_doc->Path) . '"')){
					// make fn rename to avoid redundant code
					$z = 0;
					$footext = $we_doc->Filename . '_' . $z . $we_doc->Extension;
					while(f('SELECT ID FROM ' . FILE_TABLE . ' WHERE Text="' . $GLOBALS['DB_WE']->escape($footext) . '" AND ParentID=' . intval($we_doc->ParentID))){
						$z++;
						$footext = $we_doc->Filename . '_' . $z . $we_doc->Extension;
					}
					$we_doc->Text = $footext;
					$we_doc->Filename = $we_doc->Filename . "_" . $z;
					$we_doc->Path = $we_doc->getParentPath() . (($we_doc->getParentPath() != '/') ? '/' : '') . $we_doc->Text;
				}
			}

			return $we_doc;
		}

		// check workspace for new document
		if(!$this->isParentIdOk($this->fileVars['parentID'])){
			return ['error' => 'workspace not ok',
				'success' => false,
				'weDoc' => ['id' => 0, 'path' => '']
			];
		}

		// make new we_doc
		$we_ContentType = getContentTypeFromFile($this->fileVars['weFileName']);
		$we_doc = we_contents_document::initDoc([], $we_ContentType, 0, FILE_TABLE, true);

		//FIXME: check if $we_doc exists: depends on perms!! if not: delete tmp!
		/*
		  if(!$we_doc){
		  return [
		  'error' => g_l('importFiles', '[no_perms]'),
		  'success' => false,
		  'weDoc' => ''
		  );
		  }
		 */

		// set filename, ext and path
		$filename = we_import_functions::correctFilename($this->fileVars['weFileName']);
		$matches = [];
		preg_match('#^(.*)(\..+)$#', $filename, $matches);
		if(!$matches){
			return ['error' => g_l('importFiles', '[save_error]'),
				'success' => false,
				'weDoc' => ''
			];
		}
		$we_doc->Filename = $matches[1];
		$we_doc->Extension = strtolower($matches[2]);
		if(!$we_doc->Filename){ // .htaccess
			$we_doc->Filename = $matches[2]; //.htaccess
			$we_doc->Extension = '';
		}
		$we_doc->Text = $we_doc->Filename . $we_doc->Extension;
		$we_doc->setParentID($this->fileVars['parentID']);
		$we_doc->Path = $we_doc->getParentPath() . (($we_doc->getParentPath() != '/') ? '/' : '') . $we_doc->Text;

		// if file exists we have to see if we should create a new one or overwrite it!
		if(($file_id = f('SELECT ID FROM ' . FILE_TABLE . ' WHERE Path="' . $GLOBALS['DB_WE']->escape($we_doc->Path) . '"'))){
			switch($this->fileVars['sameName']){
				case 'overwrite':
					$tmp = get_class($we_doc);
					$we_doc = new $tmp();
					$we_doc->initByID($file_id, FILE_TABLE);
					break;
				case 'rename':
					// make fn rename to avoid redundant code
					$z = 0;
					$footext = $we_doc->Filename . '_' . $z . $we_doc->Extension;
					while(f('SELECT ID FROM ' . FILE_TABLE . ' WHERE Text="' . $GLOBALS['DB_WE']->escape($footext) . '" AND ParentID=' . intval($this->fileVars['parentID']))){
						$z++;
						$footext = $we_doc->Filename . '_' . $z . $we_doc->Extension;
					}
					$we_doc->Text = $footext;
					$we_doc->Filename = $we_doc->Filename . "_" . $z;
					$we_doc->Path = $we_doc->getParentPath() . (($we_doc->getParentPath() != '/') ? '/' : '') . $we_doc->Text;
					break;
				default:
					return ['error' => g_l('importFiles', '[same_name]'),
						'success' => false,
						'weDoc' => ''
					];
			}
		}

		return $we_doc;
	}

	protected function writeWebeditionDocument($we_doc){
		$error = false;
		switch($we_doc->ContentType){
			case we_base_ContentTypes::IMAGE:
				if(!we_base_permission::hasPerm(we_base_perm::WORK_IMAGE_NEW)){
					$error = true;
				}
				break;
			case we_base_ContentTypes::HTML:
				if(!we_base_permission::hasPerm(we_base_perm::WORK_HTML_NEW)){
					$error = true;
				}
				break;
			case we_base_ContentTypes::FLASH:
			case we_base_ContentTypes::VIDEO:
			case we_base_ContentTypes::AUDIO:
				if(!we_base_permission::hasPerm(we_base_perm::WORK_VIDEO_NEW)){
					$error = true;
				}
				break;
			case we_base_ContentTypes::JS:
				if(!we_base_permission::hasPerm(we_base_perm::WORK_JS_NEW)){
					$error = true;
				}
				break;
			case we_base_ContentTypes::CSS:
				if(!we_base_permission::hasPerm(we_base_perm::WORK_CSS_NEW)){
					$error = true;
				}
				break;
			case we_base_ContentTypes::TEXT:
				if(!we_base_permission::hasPerm(we_base_perm::WORK_TEXT_NEW)){
					$error = true;
				}
				break;
			default:
				if(!we_base_permission::hasPerm(we_base_perm::WORK_OTHER_NEW)){
					$error = true;
				}
				break;
		}
		if($error){
			return ['filename' => $this->fileVars['weFileName'],
				'error' => g_l('importFiles', '[no_perms]'),
				'success' => false,
				'weDoc' => ''
			];
		}

		$tempFile = $_SERVER['DOCUMENT_ROOT'] . $this->fileVars['fileTemp'];
		// TODO: there are more bad combinations to consider
		if($we_doc->ContentType === we_base_ContentTypes::IMAGE){
			if(!$we_doc->isSvg() && !in_array(we_base_imageEdit::detect_image_type($tempFile), we_base_imageEdit::$GDIMAGE_TYPE)){
				return ['filename' => $this->fileVars['weFileName'], 'error' => g_l('alert', '[wrong_file][' . $we_doc->ContentType . ']')];
			}
		}

		//$we_doc->setElement('type', $we_doc->ContentType, "attrib");
		//$we_doc->Published = time(); // nok for parked existing docs

		if(($fh = @fopen($tempFile, 'rb'))){
			if($we_doc->isBinary()){
				$we_doc->setElement('data', $tempFile);
			} else {
				$mime = explode('/', $this->fileVars['weFileCt']);
				$we_doc->setElement('data', fread($fh, ($this->fileVars['weFileSize'] < 1 ? 1 : $this->fileVars['weFileSize'])), $mime[0]);
			}
			fclose($fh);
		} else {
			//FIXME: fopen uses less memory then gd: gd can fail (and returns 500) even if $fh = true! // ?
			//return['filename' => $_FILES['we_File']['name'], 'error' => g_l('importFiles', '[read_file_error]'));
			return ['error' => g_l('importFiles', '[read_file_error]'),
				'success' => false,
				'weDoc' => ''
			];
		}

		switch($we_doc->ContentType){
			case we_base_ContentTypes::IMAGE:
				if($this->docVars['importMetadata']){
					$we_doc->importMetaData();
				}
				if($this->docVars['focusX'] || $this->docVars['focusY']){
					$we_doc->setElement('focus', '[' . $this->docVars['focusX'] . ',' . $this->docVars['focusY'] . ']', 'input');
				}
			// no break
			case we_base_ContentTypes::FLASH:
				$we_size = $we_doc->getimagesize($tempFile);
				if(is_array($we_size) && count($we_size) >= 2){
					$we_doc->setElement('width', $we_size[0], 'attrib', we_contents_base::ETYPE_NUMBER);
					$we_doc->setElement('height', $we_size[1], 'attrib', we_contents_base::ETYPE_NUMBER);
					$we_doc->setElement('origwidth', $we_size[0], 'attrib', we_contents_base::ETYPE_NUMBER);
					$we_doc->setElement('origheight', $we_size[1], 'attrib', we_contents_base::ETYPE_NUMBER);
				}
			// no break
			default:
				$we_doc->Table = FILE_TABLE;
				$this->fileVars['weFileSize'] = $this->fileVars['weFileSize'] < 1 ? 1 : $this->fileVars['weFileSize'];
				$we_doc->setElement('filesize', $this->fileVars['weFileSize'], 'attrib', we_contents_base::ETYPE_NUMBER);

				if(we_base_permission::hasPerm(we_base_perm::WORK_CATEGORY_EDIT)){
					$we_doc->Category = isset($this->docVars['categories']) && $this->docVars['categories'] ? $this->docVars['categories'] : $we_doc->Category;
				}

				if($this->docVars['importMetadata'] && $we_doc->Extension === '.pdf'){
					$we_doc->setMetaDataFromFile($tempFile);
				}
				$we_doc->DocChanged = true;
		}

		if(in_array($we_doc->ContentType, [we_base_ContentTypes::APPLICATION,
				we_base_ContentTypes::AUDIO,
				we_base_ContentTypes::IMAGE,
				we_base_ContentTypes::VIDEO,
				we_base_ContentTypes::FLASH,
				we_base_ContentTypes::PDF,
				]
			)){
			$we_doc->IsSearchable = isset($this->docVars['isSearchable']) ? $this->docVars['isSearchable'] : $we_doc->IsSearchable;
		}

		if($we_doc->ContentType == we_base_ContentTypes::IMAGE){
			$we_doc->setElement('title', (isset($this->docVars['title']) ? $this->docVars['title'] : $we_doc->getElement('title')), 'attrib');
			$we_doc->setElement('alt', (isset($this->docVars['alt']) ? $this->docVars['alt'] : $we_doc->getElement('alt')), 'attrib');
			$we_doc->Thumbs = isset($this->docVars['thumbs']) ? $this->docVars['thumbs'] : $we_doc->Thumbs;
		}

		if($this->docVars['transaction']){
			$_SESSION['weS']['we_data']['tmpName'] = $tempFile; // what's this?
			$we_doc->saveInSession($_SESSION['weS']['we_data'][$this->docVars['transaction']]); // save the changed object in session
		} else {
			if(!$we_doc->we_save()){
				return ['error' => g_l('importFiles', '[save_error]'),
					'success' => false,
					'weDoc' => ''
				];
			}
			if(!$we_doc->we_publish()){
				return ['error' => "publish_error",
					'success' => false,
					'weDoc' => ''
				];
			}
		}

		return ['error' => [],
			'success' => true,
			'weDoc' => ['id' => $we_doc->ID,
				'path' => $we_doc->Path,
				'text' => $we_doc->Text,
				'ct' => $we_doc->ContentType,
				'table' => FILE_TABLE,
				'currentID' => $we_doc->ID,
				'currentPath' => $we_doc->Path,
				'currentText' => $we_doc->Text,
				'currentType' => $we_doc->ContentType,
				'currentTable' => FILE_TABLE
			]
		];
	}

}
