<?php
/**
 * webEdition CMS
 *
 * $Rev: 8fe15ebcad9d $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2020-03-24 10:48 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * class    we_listview_shopOrder
 * @desc    class for tag <we:listview type="'>
 *
 */
class we_listview_shopOrder extends we_listview_base{
	var $Path = '';
	var $docID = 0;
	var $hidedirindex = false;

	const replArray = [
		'OrderID' => 'ID',
		'OrderIntID' => 'ID',
		'IntOrderID' => 'ID',
		'CustomerIntID' => 'customerID',
		'IntCustomerID' => 'customerID',
		'CustomerID' => 'customerID',
	];

	/**
	 * @desc    constructor of class
	 *
	 * @param   $name          string - name of listview
	 * @param   $rows          integer - number of rows to display per page
	 * @param   $order         string - field name(s) to ORDER BY
	 * @param   $desc		   string - if desc order
	 * @param   $condition	   string - condition of listview
	 * @param   $cols		   string - number of cols (default = 1)
	 * @param   $docID	   	   string - id of a document where a we:customer tag is on
	 *
	 */
	public function __construct($name, $rows, $offset, $order, $desc, $condition, $cols, $docID, $hidedirindex){
		parent::__construct($name, $rows, $offset, $order, $desc, '', false, [], $cols);
		if($GLOBALS['we']['main']['doc']->InWebEdition){
			//do nothing inside we
			return;
		}

		$this->docID = $docID;
		$this->condition = $condition;
		$this->hidedirindex = $hidedirindex;

		// IMPORTANT for seeMode !!!! #5317
		$this->LastDocPath = (isset($_SESSION['weS']['last_webEdition_document'])) ? $_SESSION['weS']['last_webEdition_document']['Path'] : '';
		$this->Path = $this->docID ? id_to_path($this->docID, FILE_TABLE, $this->DB_WE) : (isset($GLOBALS['we_doc']) ? $GLOBALS['we_doc']->Path : '');
		$this->condition = strtr($this->condition, self::replArray);
		$this->order = strtr($this->order, self::replArray);

		if($this->order){
			$orderstring = ' ORDER BY ' . $this->order . ' ' .
				($this->desc && (!preg_match('|.+ desc$|i', $this->order)) ? ' DESC' : '');
		} else {
			$orderstring = ' ORDER BY ID' . ($this->desc ? ' DESC' : '');
		}

		$where = ($this->condition ? (' WHERE ' . $this->condition) : '');

		$this->anz_all = f('SELECT COUNT(1) FROM ' . SHOP_ORDER_TABLE . $where, '', $this->DB_WE);
		$format = [];
		foreach(we_shop_statusMails::$BaseDateFields as $field){
			$format[] = 'UNIX_TIMESTAMP(' . $field . ') AS ' . $field;
		}

		$this->DB_WE->query('SELECT
	ID AS orderid,
	ID AS ' . self::ID_FIELD . ',
	customerID,
	customerID AS ' . self::CUSTOMER_FIELD . ',
	customOrderNo,
	customFields,
	pricesNet AS shopPriceIsNet,
	calcVat AS shopCalcVat,
	priceName AS shopPricename,
	shippingCost AS Shipping_costs,
	shippingNet AS Shipping_isNet,
	shippingVat AS Shipping_vatRate,
	customerData,
	' . implode(',', $format) .
			' FROM ' . SHOP_ORDER_TABLE . $where . ' ' . $orderstring . ' ' . (($this->maxItemsPerPage > 0) ? (' LIMIT ' . $this->start . ',' . intval($this->maxItemsPerPage)) : ''));
		$this->anz = $this->DB_WE->num_rows();
	}

	public function next_record(){
		$ret = $this->DB_WE->next_record(MYSQLI_ASSOC);
		if($ret){
			$custFields = array_change_key_case(we_unserialize($this->DB_WE->f('customFields')));
			$customerData = array_change_key_case(we_unserialize($this->DB_WE->f('customerData')));

			//get all missing date fields
			$this->Record = array_merge(
				//order
				array_change_key_case($this->DB_WE->getRecord()),
				//dates
													array_change_key_case($GLOBALS['DB_WE']->getAllFirstq('SELECT type,UNIX_TIMESTAMP(date) FROM ' . SHOP_ORDER_DATES_TABLE . ' WHERE ID=' . $this->DB_WE->f(self::ID_FIELD), false)),
				//basic stuff
																																																																																			[
					self::PATH_FIELD => $this->Path . '?we_orderid=' . $this->DB_WE->f('orderid'),
					self::TEXT_FIELD => '',
					self::LASTPATH_FIELD => $this->LastDocPath . '?we_orderid=' . $this->DB_WE->f('orderid')
				],
				//custom fields
																																									 $custFields,
				//customer
																																									 self::preFixArrayKeys($customerData)
			);
			unset($this->Record['customFields'], $this->Record['customerData']);

			$this->count++;
			return true;
		}
		$this->stop_next_row = $this->shouldPrintEndTR();
		if($this->cols && ($this->count <= $this->maxItemsPerPage) && !$this->stop_next_row){
			$this->Record = [];
			$this->count++;
			return true;
		}

		return false;
	}

	public static function preFixArrayKeys(array $array){
		return array_combine(
			array_map(function($k){
				return self::CUSTOMER_PREFIX . $k;
			}, array_keys($array)), $array);
	}

	function f($key, $stripWE = false){
		return parent::f($key, $stripWE);
	}

}
