<?php

/**
 * webEdition CMS
 *
 * $Rev: 096b1a1b079c $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-10-08 17:38 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_navigation_frames extends we_modules_frame{
	var $Model;
	public $Table = NAVIGATION_TABLE;
	private $jsVars = [];

	const TAB_PROPERTIES = '1'; //make sure to keep 1
	const TAB_CONTENT = '2';
	const TAB_CUSTOMER = '3';
	const TAB_PREVIEW = 'preview';

	function __construct($frameset){
		$frameset = WEBEDITION_DIR . 'we_showMod.php?mod=navigation';
		parent::__construct($frameset);
		$this->module = we_modules_info::NAVIGATION;
		$this->showTreeFooter = true;

		$this->Tree = new we_navigation_tree($this->jsCmd);
		$this->View = new we_navigation_view();
		$this->Model = &$this->View->Model;
	}

	function getHTML($what = '', $mode = '', $step = 0){
		switch($what){
			case 'preview' :
				return we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR) ? $this->getHTMLEditorBody() : '';
			case 'previewIframe' :
				return we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR) ? $this->getHTMLEditorPreviewIframe() : '';
			case 'fields' :
				return $this->getHTMLFieldSelector();
			case 'dyn_preview' :
				return $this->getHTMLDynPreview();
			case 'frameset':
				return $this->getHTMLFrameset($this->Tree->getJSTreeCode(), (($tab = we_base_request::_(we_base_request::STRING, 'tab')) !== false ? '&tab=' . $tab : '' ) . (($sid = we_base_request::_(we_base_request::STRING, 'sid', false)) !== false ? '&sid=' . $sid : ''));
			default:
				return parent::getHTML($what, $mode, $step);
		}
	}

	function getHTMLSearch(){
		$keyword = we_base_request::_(we_base_request::RAW, 'keyword', '');
		$arr = explode(' ', strToLower($keyword));
		$DB_WE = $GLOBALS['DB_WE'];

		$condition = [get_ws_paths(NAVIGATION_TABLE, true, true)];
		foreach($arr as $value){
			$value = $DB_WE->escape($value);
			$condition[] = '(Path LIKE "%' . $value . '%" OR Text LIKE "%' . $value . '%" OR Display LIKE "%' . $value . '%")';
		}

		$entries = $DB_WE->getAllFirstq('SELECT ID,Path FROM ' . NAVIGATION_TABLE . ' WHERE ' . implode(' AND ', $condition) . ' ORDER BY Path', false);

		$select = we_html_element::div(['style' => 'background-color:white;width:520px;height:220px;']);
		if($entries){
			$select = '<select name="search_results" size="20" style="width:520px;height:220px;" ondblclick="we_cmd(\'openSelectionToEdit\');">';
			foreach($entries as $id => $path){
				$select .= '<option value="' . $id . '">' . $path . '</option>';
			}
			$select .= '</select>';
		}

		$buttons = we_html_button::position_yes_no_cancel(
				we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:we_cmd('openSelectionToEdit');"), null, we_html_button::create_button(we_html_button::CANCEL, "javascript:self.close();")
		);

		$content = we_html_tools::formElementTable(g_l('modules_users', '[search_for]'), we_html_tools::textInput('keyword', $keyword, '', [], 'text', 485), we_html_button::create_button(we_html_button::SEARCH, "javascript:document.we_form.submit();")
			) . we_html_element::div(['style' => 'height:20px;']) .
			we_html_tools::formElementTable(g_l('modules_users', '[search_result]'), $select);

		return $this->getHTMLDocument(we_html_element::body(['class' => 'weEditorBody', 'style' => 'margin:10px 20px;'], we_html_element::form(['name' => 'we_form',
						'method' => 'post'], we_html_element::hiddens([
							'mod' => 'navigation',
							'pnt' => 'search']) .
						we_html_tools::dialogLayout($content, g_l('modules_users', '[search]'), $buttons))
				), we_html_element::jsScript(WE_JS_MODULES_DIR . 'navigation/navigationSearch.js'));
	}

	protected function getHTMLTreeFooter(){
		return we_html_element::div(['id' => 'infoField', 'style' => 'display:none;']) . $this->getHTMLSearchTreeFooter();
	}

	protected function getHTMLCmd(){
		if(($pid = we_base_request::_(we_base_request::INT, 'pid')) === false){
			return $this->getHTMLDocument(we_html_element::body(), $this->jsCmd->getCmds());
		}

		$openFolders = we_base_request::_(we_base_request::INTLISTA, "open", []);
		$offset = we_base_request::_(we_base_request::INT, "offset", 0);
		$this->jsCmd->addCmd('loadTree', ['clear' => !$pid, 'items' => we_navigation_tree::getItems($pid, $offset, $this->Tree->default_segment, false, $openFolders)]);

		return $this->getHTMLDocument(we_html_element::body([], we_html_element::form(['name' => 'we_form'], we_html_element::hiddens([
							'pnt' => 'cmd',
							'cmd' => 'no_cmd'])
					)
		));
	}

	public function getHTMLDocumentHeader($charset = ''){
		return parent::getHTMLDocumentHeader($this->Model->Charset);
	}

	/**
	 * Frame for tabs
	 *
	 * @return string
	 */
	protected function getHTMLEditorHeader($mode = 0){
		if(we_base_request::_(we_base_request::BOOL, 'home')){
			return parent::getHTMLEditorHeader(0);
		}

		$we_tabs = new we_gui_tabs();

		$we_tabs->addTab('', we_gui_tabs::WE_ICON_PROPERTIES, false, self::TAB_PROPERTIES, ['id' => 'tab_' . self::TAB_PROPERTIES, 'title' => g_l('navigation', '[property]')]);
		if($this->Model->IsFolder && we_base_permission::hasPerm(we_base_perm::NAVIGATION_EDIT_DYNAMIC)){
			$we_tabs->addTab('', we_gui_tabs::WE_ICON_CONTENT, false, self::TAB_CONTENT, ['id' => 'tab_' . self::TAB_CONTENT, 'title' => g_l('navigation', '[content]')]);
		}

		if(we_modules_info::isActive(we_modules_info::CUSTOMER) && we_base_permission::hasPerm(we_base_perm::CUSTOMER_CAN_EDIT_CUSTOMERFILTER)){
			$we_tabs->addTab('', we_gui_tabs::WE_ICON_CUSTOMER_FILTER, false, self::TAB_CUSTOMER, ['id' => 'tab_' . self::TAB_CUSTOMER, 'title' => g_l('navigation', '[customers]')]);
		}

		if($this->Model->IsFolder && we_base_permission::hasPerm(we_base_perm::ADMINISTRATOR)){
			$we_tabs->addTab('', we_gui_tabs::WE_ICON_PREVIEW, false, "'" . self::TAB_PREVIEW . "'", ['id' => 'tab_' . self::TAB_PREVIEW, 'title' => g_l('navigation', '[preview]')]);
		}

		return $this->getHTMLDocument(we_html_element::body([
					'id' => "eHeaderBody",
					'onload' => "setInitialTabs(" . intval($this->Model->ID) . ',' . intval($this->Model->IsFolder) . ");initNavHeader();",
					'onresize' => "weTabs.setFrameSize()"
					], we_html_element::div(['id' => 'main'], we_html_element::div(['id' => 'headrow'], '&nbsp;' .
							we_html_element::b(g_l('navigation', ($this->Model->IsFolder ? '[group]' : '[entry]'))) . ':&nbsp;' .
							str_replace('&amp;', '&', $this->Model->Text) . ' (ID: ' . we_html_element::span(['id' => 'h_id'], $this->Model->ID) . ') ' .
							we_html_element::div(['id' => 'mark'], '<i class="fa fa-asterisk modified"></i>')) .
						$we_tabs->getHTML())
				), we_html_element::cssLink(CSS_DIR . 'we_tab.css') .
				we_html_element::jsScript(JS_DIR . 'initTabs.js') .
				we_html_element::jsScript(WE_JS_MODULES_DIR . 'navigation/navigation_view_prop.js') .
				$this->jsCmd->getCmds()
		);
	}

	protected function getHTMLEditorBody(){
		if(we_base_request::_(we_base_request::BOOL, "home")){
			return $this->View->getHomeScreen();
		}

		return $this->getHTMLDocument(we_html_element::body(['class' => 'weEditorBody', 'onload' => 'loaded=1;'], we_html_element::form(['name' => 'we_form', 'onsubmit' => 'return false'], $this->getHTMLProperties())), we_html_element::jsScript(JS_DIR . 'multiIconBox.js') . $this->View->getJSProperty($this->jsVars) .
				$this->jsCmd->getCmds());
	}

	private function getHTMLGeneral(){
		$table = new we_html_table(['class' => 'default', 'style' => 'margin-top: 5px;'], 1, 2);

		$parentid = (!empty($this->Model->Text) && !empty($this->Model->ID) ?
			f('SELECT ParentID FROM ' . NAVIGATION_TABLE . ' WHERE ID=' . intval($this->Model->ID), '', $this->db) :
			(we_base_request::_(we_base_request::STRING, 'presetFolder') ?
			$this->Model->ParentID :
			(($wq = we_navigation_navigation::getWSQuery()) ?
			f('SELECT ID FROM ' . NAVIGATION_TABLE . ' WHERE IsFolder=1 ' . $wq . ' ORDER BY Path LIMIT 1', '', $this->db) :
			0)
			));

		if($this->Model->ID){
			$table->setColContent(0, 0, we_html_element::hidden('Ordn', ($this->Model->Ordn)) .
				we_html_tools::select('Position', $this->View->getEditNaviPosition(), 1, $this->Model->Ordn, false, ['onchange' => 'we_cmd(\'move_abs\',this.value);'])
			);

			$num = f('SELECT COUNT(ID) FROM ' . NAVIGATION_TABLE . ' WHERE ParentID=' . intval($parentid));

			$table->setColContent(0, 1, we_html_element::span(['style' => 'margin-left: 15px'], we_html_button::create_button(we_html_button::DIRUP, 'javascript:we_cmd("move_up");', '', '', (($this->Model->Ordn > 0) ? false : true), false) .
					we_html_button::create_button(we_html_button::DIRDOWN, 'javascript:we_cmd("move_down");', '', '', (($this->Model->Ordn < ($num - 1)) ? false : true), false)));
		} else {
			$table->setColContent(0, 0, we_html_element::hiddens(['name' => 'Ordn', 'value' => -1]));
		}

		return [
			[
				'headline' => g_l('navigation', '[general]'),
				'html' => we_html_element::hidden('newone', ($this->Model->ID == 0 ? 1 : 0)) .
				we_html_tools::formElementTable(g_l('navigation', '[name]'), we_html_tools::textInput('Text', strtr($this->Model->Text, array_flip(get_html_translation_table(HTML_SPECIALCHARS))), '', [
						'style' => "width: 520px;", 'onchange' => "we_cmd('setHot');"])) .
				we_html_tools::formElementTable(g_l('navigation', '[display]'), we_html_tools::textInput('Display', $this->Model->Display, '', ['style' => "width: 520px;", 'onchange' => "we_cmd('setHot');"])) .
				$this->getHTMLChooser(g_l('navigation', '[group]'), NAVIGATION_TABLE, 0, 'ParentID', $parentid, 'ParentPath', 'setHot', we_base_ContentTypes::FOLDER, ($this->Model->IsFolder == 0 && $this->Model->Depended == 1)),
				'icon' => we_html_multiIconBox::PROP_PATH,
				'noline' => true
			], [
				'headline' => g_l('navigation', '[order]'),
				'html' => $table->getHtml(),
				'icon' => we_html_multiIconBox::PROP_SORT,
				'noline' => true
			],
		];
	}

	private function getHTMLImage(){
		return [
			[
				'headline' => g_l('navigation', '[icon]'),
				'html' => $this->getHTMLChooser('', FILE_TABLE, 0, 'IconID', $this->Model->IconID, 'IconPath', 'setHot', we_base_ContentTypes::IMAGE, false, true, 'folder,' . we_base_ContentTypes::IMAGE),
				'icon' => we_html_multiIconBox::PROP_IMAGE,
				'noline' => true
			], [
				'headline' => g_l('navigation', '[icon_properties]'),
				'html' => $this->getHTMLImageAttributes(),
				'fold' => true,
				'space' => we_html_multiIconBox::SPACE_ICON,
				'rightHeadline' => true,
				'noline' => true
			],
		];
	}

	private function getHTMLPropertiesItem(){
		$seltypeStat = [
			we_navigation_navigation::STYPE_DOCLINK => g_l('navigation', '[docLink]'),
			we_navigation_navigation::STYPE_URLLINK => g_l('navigation', '[urlLink]')
		];

		$seltypeDyn = [we_navigation_navigation::DYN_DOCTYPE => g_l('navigation', '[documents]')];

		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$seltypeDyn[we_navigation_navigation::DYN_CLASS] = g_l('navigation', '[objects]');
			$seltypeStat[we_navigation_navigation::STYPE_OBJLINK] = g_l('navigation', '[objLink]');
		}
		if(we_modules_info::isActive(we_modules_info::CATEGORY)){
			$seltypeDyn[we_navigation_navigation::DYN_CATEGORY] = g_l('navigation', '[categories]');
			$seltypeStat[we_navigation_navigation::STYPE_CATLINK] = g_l('navigation', '[catLink]');
		}

		switch($this->Model->Selection){
			case we_navigation_navigation::SELECTION_DYNAMIC:
				$seltype = $seltypeDyn;
				break;
			default:
				$seltype = $seltypeStat;
		}

		$selection_block = we_html_tools::select('SelectionTypeDyn', $seltypeDyn, 1, 0, false, ['style' => 'display:none']) .
			we_html_tools::select('SelectionTypeStat', $seltypeStat, 1, 0, false, ['style' => 'display:none']) .
			($this->Model->Depended == 1 ? $this->getHTMLDependedProfile() :
			we_html_element::hiddens([
				'CategoriesControl' => we_base_request::_(we_base_request::STRING, 'CategoriesControl', 0),
				'SortControl' => we_base_request::_(we_base_request::STRING, 'SortControl', 0),
			]) .
			(we_base_permission::hasPerm(we_base_perm::NAVIGATION_EDIT_DYNAMIC) ?
			we_html_tools::select('Selection', [
				we_navigation_navigation::SELECTION_DYNAMIC => g_l('navigation', '[dyn_selection]'),
				we_navigation_navigation::SELECTION_STATIC => g_l('navigation', '[stat_selection]')
				], 1, $this->Model->Selection, false, ['onchange' => "we_cmd('setSelectionMode', this);"], true, 520) . '<br/>' :
			we_html_element::hiddens([
				'Selection' => $this->Model->Selection,
			]) ) .
			we_html_tools::select('SelectionType', $seltype, 1, $this->Model->SelectionType, false, ['onchange' => "we_cmd('setSelectionType', this.value);",
				'style' => 'width: 520px; margin-top: 5px;'], true, 520) .
			we_html_element::div(['id' => 'dynamicDiv', 'style' => ($this->Model->Selection === we_navigation_navigation::SELECTION_DYNAMIC ? 'display: block;' : 'display: none;')], $this->getHTMLDynamic()) .
			we_html_element::div(['id' => 'staticDiv', 'style' => ($this->Model->Selection === we_navigation_navigation::SELECTION_STATIC ? 'display: block;' : 'display: none;')], we_html_element::div([
					'id' => 'staticSelect', 'style' => ($this->Model->SelectionType !== we_navigation_navigation::STYPE_URLLINK ? 'display: block;' : 'display: none;')], $this->getHTMLStatic()) .
				we_html_element::div(['id' => 'staticUrl', 'style' => (($this->Model->SelectionType == we_navigation_navigation::STYPE_CATLINK || $this->Model->SelectionType === we_navigation_navigation::STYPE_URLLINK) ? 'display: block;' : 'display: none;') . ';margin-top:5px;'], $this->getHTMLLink()) .
				we_html_element::div(['style' => 'margin-top:5px;'], we_html_tools::formElementTable(g_l('navigation', '[parameter]'), we_html_tools::textInput('Parameter', $this->Model->Parameter, '', [
							'onchange' => "we_cmd('setHot');"], 'text', 520, 0)))
			)
			);

		return [
			[
				'headline' => g_l('navigation', '[selection]'),
				'html' => $selection_block,
				'icon' => we_html_multiIconBox::PROP_LINK,
				'noline' => true
		]];
	}

	private function getHTMLPropertiesGroup(){
		$weSuggest = & we_gui_suggest::getInstance();

		$cmd1 = "document.we_form.elements.LinkID.value";

		$cmd_obj = we_modules_info::isActive(we_modules_info::OBJECT) ? "javascript:we_cmd('we_selector_document',document.we_form.elements.LinkID.value,WE().consts.tables.OBJECT_FILES_TABLE,'LinkID','LinkPath','populateFolderWs','','0',WE().consts.contentTypes.OBJECT_FILE,0)" : '';

		$button_doc = we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',document.we_form.elements.LinkID.value,WE().consts.tables.FILE_TABLE,'LinkID','LinkPath','','','0','',0)") .
			we_html_button::create_button(we_html_button::OPEN_TO_EDIT, 'javascript:WE().layout.openToEdit("' . FILE_TABLE . '",' . $cmd1 . ',"")', '', '', false);
		$button_obj = we_html_button::create_button(we_html_button::SELECT, $cmd_obj) .
			(we_modules_info::isActive(we_modules_info::OBJECT) ? we_html_button::create_button(we_html_button::OPEN_TO_EDIT, 'javascript:WE().layout.openToEdit("' . OBJECT_FILES_TABLE . '",' . $cmd1 . ',"")', '', '', false) : '');

		$buttons = we_html_element::div(['id' => "docFolderLink", 'style' => 'display: ' . ($this->Model->SelectionType === we_navigation_navigation::STYPE_DOCLINK ? 'inline' : 'none')], $button_doc) .
			we_html_element::div(['id' => "objFolderLink", 'style' => 'display: ' . ($this->Model->SelectionType === we_navigation_navigation::STYPE_OBJLINK ? 'inline' : 'none')], $button_obj);
		$path = ($this->Model->LinkID == 0 ?
			'' :
			id_to_path($this->Model->LinkID, ($this->Model->SelectionType === we_navigation_navigation::STYPE_DOCLINK ? FILE_TABLE : (we_modules_info::isActive(we_modules_info::OBJECT) && $this->Model->SelectionType === we_navigation_navigation::STYPE_OBJLINK ? OBJECT_FILES_TABLE : FILE_TABLE))));

		$seltype = [
			we_navigation_navigation::STYPE_DOCLINK => g_l('navigation', '[docLink]'),
			we_navigation_navigation::STYPE_URLLINK => g_l('navigation', '[urlLink]')
		];
		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$seltype[we_navigation_navigation::STYPE_OBJLINK] = g_l('navigation', '[objLink]');
		}

		$weSuggest->setAcId('LinkPath');
		$weSuggest->setContentType(
			$this->Model->SelectionType === we_navigation_navigation::STYPE_DOCLINK ?
				array_merge(we_base_ContentTypes::LINK_ABLE, [we_base_ContentTypes::FOLDER]) :
				(we_modules_info::isActive(we_modules_info::OBJECT) && $this->Model->SelectionType === we_navigation_navigation::STYPE_OBJLINK ?
					[we_base_ContentTypes::FOLDER, we_base_ContentTypes::OBJECT_FILE] :
					array_merge(we_base_ContentTypes::LINK_ABLE, [we_base_ContentTypes::FOLDER]))
		);
		$weSuggest->setInput('LinkPath', $path, [], false, true);
		$weSuggest->setMaxResults(50);
		$weSuggest->setResult('LinkID', $this->Model->LinkID);
		$weSuggest->setSelector(we_gui_suggest::DocSelector);
		$weSuggest->setTable($this->Model->SelectionType === we_navigation_navigation::STYPE_DOCLINK ? FILE_TABLE : ($this->Model->SelectionType === we_navigation_navigation::STYPE_OBJLINK ? OBJECT_FILES_TABLE : FILE_TABLE));
		$weSuggest->setWidth(330);
		$weSuggest->setSelectButton($buttons);
		$weSuggest->setTrashButton(we_html_button::create_button(we_html_button::TRASH, 'javascript:document.we_form.elements.LinkID.value=0;document.we_form.elements.LinkPath.value="";', ''));

		$weAcSelector = $weSuggest->getHTML();

		$selection = we_html_tools::select('SelectionType', $seltype, 1, $this->Model->SelectionType, false, ['onchange' => "onFolderSelectionChangeJS(this.value);setFolderSelection(this.value); we_cmd('setHot');",
				'style' => 'width: 520px; margin-top: 5px;'], true, 520) .
			we_html_element::div(['id' => "folderSelectionDiv", 'style' => 'display: ' . ($this->Model->SelectionType != we_navigation_navigation::STYPE_URLLINK ? 'block' : 'none') . ';margin-top: 5px;'], $weAcSelector) .
			we_html_element::div(['id' => "folderUrlDiv", 'style' => 'display: ' . ($this->Model->SelectionType == we_navigation_navigation::STYPE_URLLINK ? 'block' : 'none') . ';margin-top: 5px;'], we_html_tools::textInput('Url', $this->Model->Url, '', [
					'onchange' => "we_cmd('setHot');"], 'text', 520, 0)) .
			$this->getHTMLWorkspace('object', 0, 'WorkspaceID') . we_html_tools::formElementTable(g_l('navigation', '[parameter]'), we_html_tools::textInput('Parameter', $this->Model->Parameter, '', [
					'onchange' => "we_cmd('setHot');"], 'text', 520) . '<br/>' .
				we_html_forms::checkboxWithHidden($this->Model->CurrentOnUrlPar, 'CurrentOnUrlPar', g_l('navigation', '[current_on_urlpar]'), false, "defaultfont", "we_cmd('setHot');"));

		return [[
			'headline' => g_l('navigation', '[selection]'),
			'html' => $selection,
			'icon' => we_html_multiIconBox::PROP_LINK,
			'noline' => true
			],
		];
	}

	private function getHTMLPropertiesGroupMore(){
		$parts = [];
		if(function_exists('mb_convert_encoding')){
			$parts[] = [
				'headline' => g_l('navigation', '[charset]'),
				'html' => we_html_tools::alertAttentionBox(g_l('navigation', '[charset_desc]'), we_html_tools::TYPE_INFO, 520) . $this->getHTMLCharsetTable(),
				'icon' => we_html_multiIconBox::PROP_CHARSET,
				'fold' => true,
				'noline' => true
			];
		}

		// COPY FOLDER
		$disabled = ($this->Model->isnew ? ' ' . g_l('weClass', '[availableAfterSave]') : '');

		$parts[] = [
			'headline' => g_l('weClass', '[copyFolder]'),
			'html' => ($disabled ? we_html_tools::alertAttentionBox($disabled, we_html_tools::TYPE_INFO, 400, true, 0) : we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_navigation_dirSelector',document.we_form.CopyFolderID.value,'CopyFolderID','CopyFolderPath','copyNaviFolder')")) .
			we_html_element::hiddens([
				'CopyFolderID' => '',
				'CopyFolderPath' => ''
			]),
			'icon' => we_html_multiIconBox::PROP_COPY,
			'noline' => true
		];

		return $parts;
	}

	private function getHTMLDependedProfile(){
		switch($this->Model->Selection){
			case we_navigation_navigation::SELECTION_DYNAMIC:
				$seltype = [we_navigation_navigation::DYN_DOCTYPE => g_l('navigation', '[documents]')];
				if(we_modules_info::isActive(we_modules_info::OBJECT)){
					$seltype[we_navigation_navigation::DYN_CLASS] = g_l('navigation', '[objects]');
				}
				break;
			default:
				$seltype = [we_navigation_navigation::STYPE_DOCLINK => g_l('navigation', '[docLink]')];
				if(we_modules_info::isActive(we_modules_info::OBJECT)){
					$seltype[we_navigation_navigation::STYPE_OBJLINK] = g_l('navigation', '[objLink]');
				}
		}

		$table = new we_html_table(['width' => 520, 'class' => 'default defaultfont'], 5, 2);

		$table->setColContent(0, 0, g_l('navigation', '[stat_selection]'));
		$table->setColContent(1, 0, g_l('navigation', ($this->Model->SelectionType === we_navigation_navigation::STYPE_CATLINK ? '[catLink]' : ($this->Model->SelectionType === we_navigation_navigation::STYPE_DOCLINK ? '[docLink]' : '[objLink]'))) . ':');
		$table->setColContent(1, 1, id_to_path($this->Model->LinkID, ($this->Model->SelectionType === we_navigation_navigation::STYPE_CATLINK ? CATEGORY_TABLE : ($this->Model->SelectionType === we_navigation_navigation::STYPE_DOCLINK ? FILE_TABLE : OBJECT_FILES_TABLE))));

		if(!empty($this->Model->Url) && $this->Model->Url != we_base_link::EMPTY_EXT){
			$table->setColContent(2, 0, g_l('navigation', '[linkSelection]') . ':');
			$table->setColContent(2, 1, $this->Model->Url);
		} elseif(!empty($this->Model->UrlID) && $this->Model->UrlID){
			$table->setColContent(2, 0, g_l('navigation', '[linkSelection]') . ':');
			$table->setColContent(2, 1, id_to_path($this->Model->UrlID));
		}

		if($this->Model->SelectionType === we_navigation_navigation::STYPE_CATLINK){
			$table->setColContent(3, 0, g_l('navigation', '[catParameter]') . ':');
			$table->setColContent(3, 1, $this->Model->CatParameter);
		}

		if(!empty($this->Model->Parameter)){
			$table->setColContent(4, 0, g_l('navigation', '[parameter]') . ':');
			$table->setColContent(4, 1, $this->Model->Parameter);
		}

		return $table->getHtml();
	}

	private function getHTMLDynamic(){
		$dtq = we_doctype_docTypes::getDoctypeQuery($this->db);
		$docTypes = [g_l('navigation', '[no_entry]')] + $this->db->getAllFirstq('SELECT dt.ID,dt.DocType FROM ' . DOC_TYPES_TABLE . ' dt LEFT JOIN ' . FILE_TABLE . ' dtf ON dt.ParentFileID=dtf.ID ' . $dtq['join'] . ' WHERE ' . $dtq['where'], false);

		$classID2Name = $classID2Dir = $classDirs = [];
		$this->jsVars = [
			'classDirs' => [],
			'classPaths' => [],
			'hasClassSubDirs' => []
		];
		$allowedClasses = we_users_util::getAllowedClasses($this->db);

		$firstClass = 0;
		if(we_modules_info::isActive(we_modules_info::OBJECT) && $allowedClasses){
			$this->db->query('SELECT DISTINCT o.ID,o.Text,o.Path,`of`.ID AS classDirID FROM ' . OBJECT_TABLE . ' o JOIN ' . OBJECT_FILES_TABLE . ' `of` ON (o.ID=`of`.TableID) WHERE `of`.IsClassFolder=1 AND o.ID IN(' . implode(',', $allowedClasses) . ')');
			while($this->db->next_record()){
				if(!$firstClass){
					$firstClass = $this->db->f('ID');
				}
				$classID2Name[$this->db->f('ID')] = $this->db->f('Text');
				$classID2Dir[$this->db->f('classDirID')] = $this->db->f('ID');
				$classDirs[] = $this->db->f('classDirID');

				$this->jsVars['hasClassSubDirs'][$this->db->f('ID')] = false;
				$this->jsVars['classDirs'][$this->db->f('ID')] = $this->db->f('classDirID');
				$this->jsVars['classPaths'][$this->db->f('ID')] = $this->db->f('Path');
			}
			asort($classID2Name);
			if($classDirs){
				$this->db->query('SELECT TableID FROM ' . OBJECT_FILES_TABLE . ' WHERE TableID IN (' . implode(',', $allowedClasses) . ') AND IsFolder=1 GROUP BY TableID');
				while($this->db->next_record()){
					$this->jsVars['hasClassSubDirs'][$this->db->f('TableID')] = true;
				}
			}
		}

		/* $wsid = ($this->Model->SelectionType == we_navigation_navigation::STYPE_OBJLINK && $this->Model->LinkID ?
		  we_navigation_dynList::getWorkspacesForObject($this->Model->LinkID) :
		  []);
		 */

		$sortVal = isset($this->Model->Sort[0]['field']) ? $this->Model->Sort[0]['field'] : '';
		$sortOrder = isset($this->Model->Sort[0]['order']) ? $this->Model->Sort[0]['order'] : 'ASC';

		$sortSelect = we_html_tools::select('SortOrder', ['ASC' => g_l('navigation', '[ascending]'), 'DESC' => g_l('navigation', '[descending]')], 1, $sortOrder, false, [
				'onchange' => "we_cmd('setHot');"], true, 120);

		$selectionType = $this->Model->IsFolder ? $this->Model->DynamicSelection : $this->Model->SelectionType;

		return we_html_element::div(['id' => 'doctype', 'style' => ($this->Model->Selection === we_navigation_navigation::SELECTION_DYNAMIC && $selectionType === we_navigation_navigation::DYN_DOCTYPE ? 'display: block' : 'display: none') . ';width: 520px;margin-top:5px;'], we_html_tools::formElementTable(g_l('navigation', '[doctype]'), we_html_tools::select('DocTypeID', $docTypes, 1, $this->Model->DocTypeID, false, [
						'onchange' => "clearFields(); we_cmd('setHot');"], true, 520))) .
			we_html_element::div(['id' => 'classname', 'style' => ($this->Model->Selection === we_navigation_navigation::SELECTION_DYNAMIC && $selectionType === we_navigation_navigation::DYN_CLASS ? 'display: block' : 'display: none') . '; width: 520px;margin-top:5px;'], (we_modules_info::isActive(we_modules_info::OBJECT) ? we_html_tools::formElementTable(g_l('navigation', '[class]'), we_html_tools::select('ClassID', $classID2Name, 1, $this->Model->ClassID, false, [
						'onchange' => "clearFields();onSelectionClassChangeJS(this.value);"], true, 520)) . $this->getHTMLWorkspace('class', $firstClass) : '')) . we_html_element::div([
				'id' => 'fieldChooser', 'style' => ($selectionType != we_navigation_navigation::DYN_CATEGORY ? 'display: block' : 'display: none') . '; width: 520px;margin-top: 5px;'], $this->getHTMLFieldChooser(g_l('navigation', '[title_field]'), 'TitleField', $this->Model->TitleField, 'putTitleField', $selectionType)) .
			$this->getHTMLDirSelector() .
			we_html_element::div(['id' => 'catSort', 'style' => ($selectionType != we_navigation_navigation::DYN_CATEGORY ? 'display: block;' : 'display: none;') . '; width: 520px;'], (we_modules_info::isActive(we_modules_info::CATEGORY) ? $this->getHTMLCategory() : '') .
				$this->getHTMLFieldChooser(g_l('navigation', '[sort]'), 'SortField', $sortVal, 'putSortField', $selectionType, $sortSelect, 120)) .
			we_html_element::div(['id' => 'dynUrl', 'style' => ($selectionType == we_navigation_navigation::DYN_CATEGORY ? 'display: block' : 'display: none') . '; width: 520px;'], $this->getHTMLLink('dynamic_')) .
			we_html_element::div(['style' => 'width: 520px;margin-top: 5px;'], we_html_tools::formElementTable(g_l('navigation', '[parameter]'), we_html_tools::textInput('dynamic_Parameter', $this->Model->Parameter, '', [
						'onchange' => "we_cmd('setHot');"], 'text', 520, 0))) .
			$this->getHTMLCount() .
			we_html_element::span(['style' => 'margin-top:20px;'], we_html_button::create_button(we_html_button::PREVIEW, 'javascript:top.content.we_cmd("dyn_preview");') . ($this->Model->hasDynChilds() ? we_html_button::create_button(we_html_button::DELETE_ALL, 'javascript:top.content.we_cmd("depopulate");') : ''));
	}

	private function getHTMLStatic($disabled = false){
		$seltype = [we_navigation_navigation::STYPE_DOCLINK => g_l('navigation', '[docLink]')];
		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$seltype[we_navigation_navigation::STYPE_OBJLINK] = g_l('navigation', '[objLink]');
		}

		$cmd_doc = "javascript:we_cmd('we_selector_document',document.we_form.elements.LinkID.value,WE().consts.tables.FILE_TABLE,'LinkID','LinkPath','enable_open_navigation_doc','','0',''," . (we_base_permission::hasPerm(we_base_perm::WORK_WORKSPACE_CAN_SELECT_OTHER_FILES) ? 0 : 1) . ")";

		$cmd_obj = we_modules_info::isActive(we_modules_info::OBJECT) ? "javascript:we_cmd('we_selector_document',document.we_form.elements.LinkID.value,WE().consts.tables.OBJECT_FILES_TABLE,'LinkID','LinkPath','populateWorkspaces','','0',''," . (we_base_permission::hasPerm(we_base_perm::OBJECTS_CAN_SELECT_OTHER_USERS) ? 0 : 1) . ")" : '';
		$cmd_cat = "javascript:we_cmd('we_selector_document',document.we_form.elements.LinkID.value,WE().consts.tables.CATEGORY_TABLE,'LinkID','LinkPath','populateText','','0')";

		$button_doc = we_html_button::create_button(we_html_button::SELECT, $cmd_doc, '', '', $disabled) .
			we_html_button::create_button(we_html_button::OPEN_TO_EDIT, 'javascript:WE().layout.openToEdit(WE().consts.tables.FILE_TABLE,document.we_form.elements.LinkID.value,"")', '', '', ($this->Model->LinkID ? false : true), false, '_navigation_doc');

		$button_obj = we_html_button::create_button(we_html_button::SELECT, $cmd_obj, '', '', $disabled) .
			(we_modules_info::isActive(we_modules_info::OBJECT) ? we_html_button::create_button(we_html_button::OPEN_TO_EDIT, 'javascript:WE().layout.openToEdit(WE().consts.tables.OBJECT_FILES_TABLE,document.we_form.elements.LinkID.value,"")', '', '', $disabled, false, '_navigation_obj') : '');

		$buttons = we_html_element::div(['id' => "docLink", 'style' => 'display: ' . ($this->Model->SelectionType === we_navigation_navigation::STYPE_DOCLINK ? 'inline' : 'none')], $button_doc) .
			we_html_element::div(['id' => "objLink", 'style' => 'display: ' . ($this->Model->SelectionType === we_navigation_navigation::STYPE_OBJLINK ? 'inline' : 'none')], $button_obj) .
			(we_modules_info::isActive(we_modules_info::CATEGORY) ?
			we_html_element::div(['id' => "catLink", 'style' => 'display: ' . ($this->Model->SelectionType === we_navigation_navigation::STYPE_CATLINK ? 'inline' : 'none')], we_html_button::create_button(we_html_button::SELECT, $cmd_cat, '', '', $disabled)) :
			'');
		$path = ($this->Model->LinkID == 0 ?
			'' :
			id_to_path($this->Model->LinkID, ($this->Model->SelectionType === we_navigation_navigation::STYPE_DOCLINK ? FILE_TABLE : ($this->Model->SelectionType === we_navigation_navigation::STYPE_CATLINK ? CATEGORY_TABLE : OBJECT_FILES_TABLE )))
			);

		$weSuggest = & we_gui_suggest::getInstance();
		$weSuggest->setAcId('LinkPath', '');
		$weSuggest->setContentType(
			$this->Model->SelectionType === we_navigation_navigation::STYPE_DOCLINK ?
				array_merge(we_base_ContentTypes::LINK_ABLE, [we_base_ContentTypes::FOLDER]) :
				($this->Model->SelectionType === we_navigation_navigation::STYPE_OBJLINK ? we_base_ContentTypes::OBJECT_FILE : ''));
		$weSuggest->setInput('LinkPath', $path, [], false, true);
		$weSuggest->setMaxResults(50);
		$weSuggest->setResult('LinkID', $this->Model->LinkID);
		$weSuggest->setSelector(we_gui_suggest::DocSelector);
		$weSuggest->setTable($this->Model->SelectionType === we_navigation_navigation::STYPE_DOCLINK ? FILE_TABLE : ($this->Model->SelectionType === we_navigation_navigation::STYPE_OBJLINK ? OBJECT_FILES_TABLE : CATEGORY_TABLE));
		$weSuggest->setWidth(370);
		$weSuggest->setSelectButton($buttons);

		return we_html_element::div(['style' => "margin-top:5px"], $weSuggest->getHTML()) . $this->getHTMLWorkspace();
	}

	private function getHTMLTab2(){
		/* if($this->Model->hasDynChilds()) {
		  return $this->getHTMLContentInfo();
		  } */

		$seltype = [we_navigation_navigation::DYN_DOCTYPE => g_l('navigation', '[documents]')];

		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$seltype[we_navigation_navigation::DYN_CLASS] = g_l('navigation', '[objects]');
		}
		if(we_modules_info::isActive(we_modules_info::CATEGORY)){
			$seltype[we_navigation_navigation::DYN_CATEGORY] = g_l('navigation', '[categories]');
		}
		$selection_block = we_html_element::hiddens([
				'CategoriesControl' => we_base_request::_(we_base_request::STRING, 'CategoriesControl', 0),
				'SortControl' => we_base_request::_(we_base_request::STRING, 'SortControl', 0),
			]) .
			we_html_tools::select('Selection', [
				we_navigation_navigation::SELECTION_STATIC => g_l('navigation', '[no_dyn_content]'),
				we_navigation_navigation::SELECTION_DYNAMIC => g_l('navigation', '[dyn_content]')
				], 1, $this->Model->Selection, false, ['style' => 'width: 520px;', 'onchange' => "we_cmd('setSelectionMode', this, true);"], true, 520) .
			we_html_element::div(['id' => 'dynamicDiv', 'style' => ($this->Model->Selection === we_navigation_navigation::SELECTION_DYNAMIC ? 'display: block;' : 'display: none;') . ';margin-top:5px'], we_html_tools::select('DynamicSelection', $seltype, 1, $this->Model->DynamicSelection, false, [
					'onchange' => "we_cmd('setSelectionType', this.value);"], true, 520) .
				$this->getHTMLDynamic()) . we_html_element::div(['id' => 'staticDiv', 'style' => 'display:none;']);

		return [
			['headline' => g_l('navigation', '[content]'),
				'html' => $selection_block,
				'space' => we_html_multiIconBox::SPACE_MED
		]];
	}

	private function getHTMLEditorPreview(){
		// build the page

		return $this->View->getCommonHiddens(['pnt' => 'preview', 'tabnr' => 'preview']) . we_html_tools::dialogLayout('<table class="defaultfont" class="default">
<tr>
	<td><iframe name="preview" style="background: white; border: 1px solid black; width: 640px; height: 150px" src="' . WEBEDITION_DIR . 'we_showMod.php?mod=navigation&pnt=previewIframe"></iframe></td>
</tr>
<tr>
	<td height="30"><label for="previewCode">' . g_l('navigation', '[preview_code]') . '</label><br/></td>
<tr>
	<td>' . we_html_element::textArea(
					[
						'name' => 'previewCode',
						'id' => 'previewCode',
						'style' => 'width: 640px; height: 200px;',
						'class' => 'defaultfont'
					], $this->Model->previewCode) . '</td>
</tr>
<tr>
	<td height="10"></td>
<tr>
<tr>
	<td style="text-align:right">' .
				we_html_button::create_button(we_html_button::REFRESH, 'javascript: showPreview();') .
				we_html_button::create_button(we_html_button::RESET, 'javascript: document.getElementById("previewCode").value = "' . str_replace(["\r\n", "\n"], '\n', addslashes(str_replace('@###PARENTID###@', $this->Model->ID, we_navigation_navigation::defaultPreviewCode))) . '"; showPreview();')
				//,we_button::create_button('new_template', 'javascript: top.content.we_cmd("create_template");')
				. '</td>
</tr>
</table>', g_l('navigation', '[preview]'));
	}

	private function getHTMLEditorPreviewIframe(){
		require_once (WE_INCLUDES_PATH . 'we_tag.inc.php');

		$templateCode = $this->Model->previewCode;

		// initialize a document (only for caching needed)
		//$GLOBALS['we_doc'] = new we_webEditionDocument();

		$tp = new we_weTag_parser($templateCode);
		$tp->parseTags($templateCode);
		if(($file = we_base_file::saveTemp($templateCode))){
			include($file);
		}
	}

	function getHTMLProperties(){
		$tabNr = we_base_request::_(we_base_request::STRING, 'tabnr', self::TAB_PROPERTIES); //FIXME: due to preview - fix this as a better tab-name; replace 1-3 with consts

		if($this->Model->IsFolder == 0 && $tabNr != self::TAB_PROPERTIES && $tabNr != self::TAB_CUSTOMER){
			$tabNr = self::TAB_PROPERTIES;
		}

		$hiddens = [
			'cmd' => '',
			'pnt' => 'edbody',
			'tabnr' => $tabNr,
		];

		//$weSuggest = & weSuggest::getInstance();
		switch($tabNr){
			case self::TAB_PREVIEW:
				return $this->getHTMLEditorPreview();
			default:
				// Property tab content
				return we_html_element::div(
						['id' => 'tab1',
							'style' => ($tabNr == self::TAB_PROPERTIES ? 'display: block;' : 'display: none')
						], $this->View->getCommonHiddens($hiddens) .
						we_html_element::hiddens([
							'IsFolder' => $this->Model->IsFolder,
							'presetFolder' => we_base_request::_(we_base_request::STRING, 'presetFolder', '')]) .
						we_html_multiIconBox::getHTML('', array_merge(
								$this->getHTMLGeneral(), ($this->Model->IsFolder ?
								$this->getHTMLPropertiesGroup() :
								$this->getHTMLPropertiesItem()
								), (($this->Model->Selection == we_navigation_navigation::SELECTION_STATIC || $this->Model->IsFolder) ? [[
								'rightHeadline' => true,
								'space' => we_html_multiIconBox::SPACE_ICON,
								'headline' => g_l('navigation', '[more_attributes]'),
								'fold' => true,
								'html' => we_html_multiIconBox::getHTML('weNaviAttrib', $this->getHTMLAttributes()),
								'noline' => true,
								]] : []), ($this->Model->IsFolder ? $this->getHTMLPropertiesGroupMore() : []), $this->getHTMLImage()
							), true)
					) .
					($this->Model->IsFolder && we_base_permission::hasPerm(we_base_perm::NAVIGATION_EDIT_DYNAMIC) ?
					we_html_element::div(['id' => 'tab' . self::TAB_CONTENT, 'style' => ($tabNr == self::TAB_CONTENT ? 'display: block;' : 'display: none')], we_html_multiIconBox::getHTML('', $this->getHTMLTab2(), true)) :
					''
					) . ((we_modules_info::isActive(we_modules_info::CUSTOMER)) ?
					we_html_element::div(['id' => 'tab' . self::TAB_CUSTOMER, 'style' => ($tabNr == self::TAB_CUSTOMER ? 'display: block;' : 'display: none')], we_html_multiIconBox::getHTML('', $this->getHTMLTab3(), true)) :
					''
					);
		}
	}

	private function getHTMLFieldChooser($title, $name, $value, $cmd, $type, $extraField = '', $extraFieldWidth = 0){
		$selectionType = $this->Model->IsFolder ? $this->Model->DynamicSelection : $this->Model->SelectionType;
		$disabled = !(($selectionType === we_navigation_navigation::DYN_CLASS && $this->Model->ClassID != 0) || ($selectionType === we_navigation_navigation::DYN_DOCTYPE && $this->Model->DocTypeID != 0));
		$button = we_html_button::create_button(we_html_button::SELECT, "javascript:fieldChooserBut('" . $cmd . "');", '', '', $disabled, false, '_' . $name);
		if(!$extraField){
			$showValue = ($type === we_navigation_navigation::DYN_CLASS && stristr($value, '_')) ? substr($value, strpos($value, '_') + 1) : $value;
			return we_html_tools::formElementTable($title, we_html_element::hidden($name, $value) . we_html_tools::textInput('__' . $name, $showValue, '', ['onchange' => "setFieldValue('" . $name . "',this); we_cmd('setHot');"], 'text', 400, 0), '', $button);
		}
		$showValue = stristr($value, '_') ? substr($value, strpos($value, '_') + 1) : $value;
		return we_html_tools::formElementTable($title, we_html_element::hidden($name, $value) .
				we_html_tools::textInput('__' . $name, $showValue, '', ['onchange' => "setFieldValue('" . $name . "',this); we_cmd('setHot');"], 'text', 400 - abs($extraFieldWidth) - 8, 0), '', $extraField, $button);
	}

	private function getHTMLChooser($title, $table = FILE_TABLE, $rootDirID = 0, $IDName = 'ID', $IDValue = '', $PathName = 'Path', $cmd = '', $filter = we_base_ContentTypes::WEDOCUMENT, $disabled = false, $showtrash = false, $acCTypes = ''){
		if($IDValue == 0){
			$path = '/';
		} elseif(isset($this->Model->$IDName) && !empty($this->Model->$IDName)){
			$path = id_to_path($this->Model->$IDName, $table);
		} else {
			$path = ($IDValue !== '' ? id_to_path($IDValue, $table) : '');
		}

		if($table === NAVIGATION_TABLE){
			$cmd = "javascript:we_cmd('we_navigation_dirSelector',document.we_form.$IDName.value,'" . $IDName . "','" . $PathName . "','" . $cmd . "')";
			$selector = we_gui_suggest::DirSelector;
		} else if($filter === we_base_ContentTypes::FOLDER){
			$cmd = "javascript:we_cmd('we_selector_file',document.we_form.$IDName.value,'" . $table . "','" . $IDName . "','" . $PathName . "','" . $cmd . "','','" . $rootDirID . "')";
			$selector = we_gui_suggest::DirSelector;
		} else {
			$cmd = "javascript:we_cmd('" . ($filter == we_base_ContentTypes::IMAGE ? 'we_selector_image' : 'we_selector_document') . "',document.we_form.$IDName.value,'" . $table . "','" . $IDName . "','" . $PathName . "','" . $cmd . "','','" . $rootDirID . "','" . $filter . "'," . (we_base_permission::hasPerm(we_base_perm::WORK_WORKSPACE_CAN_SELECT_OTHER_FILES) ? 0 : 1) . ")";
			$selector = we_gui_suggest::DocSelector;
		}

		if($selector == we_gui_suggest::DocSelector){
			$path = $path === '/' ? '' : $path;
			$required = 0;
		} else {
			$required = 1;
		}

		if($showtrash){
			$button = we_html_button::create_button(we_html_button::SELECT, $cmd, '', '', $disabled) .
				we_html_button::create_button(we_html_button::TRASH, 'javascript:document.we_form.elements["' . $IDName . '"].value=0;document.we_form.elements["' . $PathName . '"].value="/";');
			$width = 157;
		} else {
			$width = 120;
			$button = we_html_button::create_button(we_html_button::SELECT, $cmd, '', '', $disabled);
		}

		$weSuggest = &we_gui_suggest::getInstance();
		$weSuggest->setAcId($PathName);
		$weSuggest->setContentType($acCTypes ?: $filter);
		$weSuggest->setInput($PathName, $path, [], false, true);
		$weSuggest->setLabel($title);
		$weSuggest->setMaxResults(50);
		$weSuggest->setRequired($required);
		$weSuggest->setResult($IDName, $IDValue);
		$weSuggest->setSelector($selector);
		$weSuggest->setTable($table);
		$weSuggest->setWidth(520 - $width);
		$weSuggest->setSelectButton($button);

		$weAcSelector = $weSuggest->getHTML();
		return (isset($weAcSelector) ?
			$weAcSelector :
			we_html_tools::formElementTable($title, we_html_tools::textInput($PathName, $path, '', ['readonly' => 'readonly'], 'text', (520 - $width), 0), we_html_element::hidden($IDName, $IDValue), $button)
			);
	}

	private function getHTMLCategory(){
		$addbut = we_html_button::create_button(we_html_button::ADD, "javascript:we_cmd('we_selector_document','',WE().consts.tables.CATEGORY_TABLE,'','','addCat');we_cmd('setHot');");
		$del_but = we_html_button::create_button(we_html_button::TRASH, "javascript:" . we_base_constants::PLACEHOLDER . "; we_cmd('setHot');");

		$this->jsCmd->addCmd('categoriesEdit', [510, $this->Model->Categories, $del_but]);

		$table = new we_html_table([
			'id' => 'CategoriesBlock',
			'style' => 'display: block;',
			'class' => 'default withSpace'
			], 3, 2
		);

		$table->setCol(0, 0, ['colspan' => 2, 'class' => 'defaultfont'], g_l('navigation', '[categories]'));
		$table->setCol(1, 0, ['colspan' => 2], we_html_element::div([
				'id' => 'categories',
				'class' => 'blockWrapper',
				'style' => 'width: 520px; height: 60px; border: #AAAAAA solid 1px;'
				]
			)
		);
		$table->setCol(2, 0, ['style' => 'text-align:left'], we_html_forms::checkboxWithHidden($this->Model->CatAnd, "CatAnd", g_l('navigation', '[catAnd]')));
		$table->setCol(2, 1, ['style' => 'text-align:right'], we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:removeAllCats()") . $addbut);

		return $table->getHtml();
	}

	private function getHTMLFieldSelector(){
		$type = we_base_request::_(we_base_request::STRING, 'type'); // doctype || class
		$selection = we_base_request::_(we_base_request::INT, 'selection'); // doctype or classid
		$cmd = we_base_request::_(we_base_request::JS, 'cmd'); // js command
		$multi = we_base_request::_(we_base_request::BOOL, 'multi'); // js command

		$fields = [];
		switch($type){
			case we_navigation_navigation::DYN_DOCTYPE://FIXME: is this called?
			case we_navigation_navigation::STYPE_DOCLINK://this is correct
				$ids = array_filter(explode(',', f('SELECT Templates FROM ' . DOC_TYPES_TABLE . ' WHERE ID=' . intval($selection))));
				foreach($ids as $templateID){
					$template = new we_template();
					$template->initByID($templateID, TEMPLATES_TABLE);
					$fields = array_merge($fields, array_keys($template->readAllVariantFields(true)));
				}
				$f = array_unique($fields);
				$fields = array_combine($f, $f);
				break;
			default:
				if(we_modules_info::isActive(we_modules_info::OBJECT)){
					$class = new we_object();
					$class->initByID($selection, OBJECT_TABLE);
					$found = array_keys($class->getAllVariantFields());
					foreach($found as $key){
						$fields[$key] = substr($key, strpos($key, '_') + 1);
					}
				}
		}
		asort($fields);
		$parts = [
			[
				'headline' => '',
				'html' => we_html_tools::select('fields', $fields, 20, '', $multi, ['style' => "width: 300px; height: 200px; margin: 5px 0 5px 0;", 'onclick' => "setTimeout(selectItem,100);"]),
			]
		];

		$body = we_html_element::body(
				['class' => 'weDialogBody', 'onload' => "loaded=1;"], we_html_element::form(['name' => 'we_form', 'onsubmit' => "return false"], we_html_multiIconBox::getHTML('', $parts, true, we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::OK, "javascript:setFields('" . $cmd . "');", '', '', true, false), null, we_html_button::create_button(we_html_button::CLOSE, 'javascript:self.close();')), -1, '', '', false, g_l('navigation', '[select_field_txt]'))));

		return $this->getHTMLDocument($body, we_html_element::jsScript(WE_JS_MODULES_DIR . 'navigation/navigation_view_prop.js'));
	}

	private function getHTMLCount(){
		return we_html_element::div(['style' => "width: 520px; margin-top: 5px"], we_html_tools::formElementTable(g_l('navigation', '[show_count]'), we_html_tools::textInput('ShowCount', $this->Model->ShowCount
						, '', ['onblur' => "var r=parseInt(this.value);if(isNaN(r)){this.value=" . $this->Model->ShowCount . ";}else{ this.value=r; we_cmd('setHot');}"], 'text', 520, 0)));
	}

	private function getHTMLDirSelector(){
		$selectionType = $this->Model->IsFolder ? $this->Model->DynamicSelection : $this->Model->SelectionType;
		$table = $selectionType === we_navigation_navigation::DYN_DOCTYPE ? FILE_TABLE :
			($selectionType === we_navigation_navigation::DYN_CLASS ? OBJECT_FILES_TABLE :
			($selectionType === we_navigation_navigation::DYN_CATEGORY ? CATEGORY_TABLE :
			FILE_TABLE));

		$rootDirID = (($ws = get_ws($table, true)) ? reset($ws) : 0);

		$button_doc = we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_directory',document.we_form.elements.FolderID.value,WE().consts.tables.FILE_TABLE,'FolderID','FolderPath','setHot','','" . $rootDirID . "')");
		$countSubDirs = 1;
		if(we_modules_info::isActive(we_modules_info::OBJECT) && ($selectionType === we_navigation_navigation::DYN_CLASS || $selectionType === we_navigation_navigation::STYPE_OBJLINK)){
			$classDirID = f('SELECT `of`.ID FROM ' . OBJECT_FILES_TABLE . ' `of` WHERE `of`.IsClassFolder=1 AND `of`.TableID=' . $this->Model->ClassID, '', $this->db);
			$countSubDirs = f('SELECT COUNT(1) FROM ' . OBJECT_FILES_TABLE . ' WHERE ParentID=' . $classDirID . ' AND IsFolder=1', '', $this->db);
		}

		$button_obj = we_modules_info::isActive(we_modules_info::OBJECT) ? we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_directory',document.we_form.elements.FolderID.value,WE().consts.tables.OBJECT_FILES_TABLE,'FolderID','FolderPath','setHot','',props.classDirs[document.we_form.elements.ClassID.options[document.we_form.elements.ClassID.selectedIndex].value])", '', "", ($countSubDirs ? false : true), false, "_XFolder") : '';
		$button_cat = we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',document.we_form.elements.FolderID.value,WE().consts.tables.CATEGORY_TABLE,'FolderID','FolderPath','','','" . $rootDirID . "')");
		$buttons = we_html_element::div(['id' => 'docFolder', 'style' => 'display: ' . (($table === FILE_TABLE) ? 'inline' : 'none')], $button_doc) .
			we_html_element::div(['id' => "objFolder", 'style' => 'display: ' . ($table === OBJECT_FILES_TABLE ? 'inline' : 'none')], $button_obj) .
			we_html_element::div(['id' => "catFolder", 'style' => 'display: ' . ($table === CATEGORY_TABLE ? 'inline' : 'none')], $button_cat);


		$path = id_to_path($this->Model->FolderID ?: $rootDirID, $table);
		$attribs = ['onchange' => "we_cmd('setHot');"];
		if(!$countSubDirs){
			$attribs['disabled'] = "disabled";
		}

		$weSuggest = & we_gui_suggest::getInstance();
		$weSuggest->setAcId('FolderPath', $table === OBJECT_FILES_TABLE ? id_to_path($this->Model->ClassID, OBJECT_FILES_TABLE) : "");
		$weSuggest->setContentType(we_base_ContentTypes::FOLDER);
		$weSuggest->setInput('FolderPath', $path, $attribs);
		$weSuggest->setMaxResults(50);
		$weSuggest->setResult('FolderID', $this->Model->FolderID ?: $rootDirID);
		$weSuggest->setSelector(we_gui_suggest::DirSelector);
		$weSuggest->setLabel(g_l('navigation', '[dir]'));
		$weSuggest->setTable($table);
		$weSuggest->setWidth(400);
		$weSuggest->setSelectButton($buttons);

		$weAcSelector = $weSuggest->getHTML();

		return we_html_element::div(['style' => 'margin-top:5px;'], $weAcSelector);
	}

	private function getHTMLDynPreview(){
		$select = new we_html_select(['size' => 20, 'style' => 'width: 420px; height: 250px; margin: 5px 0 5px 0;']);

		$items = $this->Model->getDynamicEntries();

		foreach($items as $k => $item){
			$txt = id_to_path($item['id'], ($this->Model->DynamicSelection === we_navigation_navigation::DYN_DOCTYPE) ? FILE_TABLE : ($this->Model->DynamicSelection === we_navigation_navigation::DYN_CATEGORY ? CATEGORY_TABLE : OBJECT_FILES_TABLE));
			if($item['field']){
				$opt = we_html_select::getNewOptionGroup([
						'class' => 'bold',
						'style' => 'font-style: normal; color: darkblue;',
						'label' => $item['field']
				]);
				$opt->addChild(we_html_select::getNewOption($k, $txt));
				$select->addChild($opt);
			} else {
				$select->addOption($k, $txt);
			}
		}

		return $this->getHTMLDocument(
				we_html_element::body(['class' => 'weDialogBody'], we_html_element::form([
						'name' => 'we_form', 'onsubmit' => 'return false'
						], we_html_multiIconBox::getHTML('', [
							[
								'headline' => g_l('navigation', ($this->Model->DynamicSelection === we_navigation_navigation::DYN_CATEGORY ? '[categories]' : ($this->Model->DynamicSelection === we_navigation_navigation::DYN_CLASS ? '[objects]' : '[documents]'))),
								'html' => $select->getHtml(),
							]
							], true, we_html_element::div(['style' => "float:right;"], we_html_button::create_button(we_html_button::CLOSE, 'javascript:self.close();')
							), -1, '', '', false, g_l('navigation', '[dyn_selection]')))), '');
	}

	private function getHTMLWorkspace($type = 'object', $defClassID = 0, $field = 'WorkspaceID'){
		if($type === 'class'){
			$wsid = (($this->Model->IsFolder ? $this->Model->DynamicSelection : $this->Model->SelectionType) === we_navigation_navigation::DYN_CLASS && $this->Model->ClassID ?
				we_navigation_dynList::getWorkspacesForClass($this->Model->ClassID) :
				($defClassID ?
				we_navigation_dynList::getWorkspacesForClass($defClassID) :
				[]
				));

			return we_html_element::div(['id' => "objLinkWorkspaceClass", 'style' => 'display: ' . (($this->Model->Selection == we_navigation_navigation::SELECTION_DYNAMIC) ? 'block' : 'none') . ';margin-top: 5px;'], we_html_tools::formElementTable(g_l('navigation', '[workspace]'), we_html_tools::select('WorkspaceIDClass', $wsid, 1, $this->Model->WorkspaceID, false, [
							'style' => 'width: 520px;', 'onchange' => "we_cmd('setHot');"])
			));
		}

		$wsid = ($this->Model->SelectionType === we_navigation_navigation::STYPE_OBJLINK && $this->Model->LinkID ?
			we_navigation_dynList::getWorkspacesForObject($this->Model->LinkID) : []);

		if($field === 'WorkspaceID'){
			return we_html_element::div(['id' => "objLinkWorkspace", 'style' => 'display: ' . (($this->Model->SelectionType === we_navigation_navigation::STYPE_OBJLINK && ($this->Model->WorkspaceID > -1)) ? 'block' : 'none') . ';margin-top: 5px;'], we_html_tools::formElementTable(g_l('navigation', '[workspace]'), we_html_tools::select('WorkspaceID', $wsid, 1, $this->Model->WorkspaceID, false, [
							'style' => 'width: 520px;', 'onchange' => "we_cmd('setHot');"])));
		}

		return we_html_element::div(['id' => "objLinkFolderWorkspace", 'style' => 'display: ' . (($this->Model->SelectionType === we_navigation_navigation::STYPE_OBJLINK && ($this->Model->WorkspaceID > -1)) ? 'block' : 'none') . ';margin-top: 5px;'], we_html_tools::formElementTable(g_l('navigation', '[workspace]'), we_html_tools::select('WorkspaceID', $wsid, 1, $this->Model->WorkspaceID, false, [
						'style' => 'width: 520px;', 'onchange' => "we_cmd('setHot');"])));
	}

	private function getHTMLLink($prefix = ''){
		$cmd = "javascript:we_cmd('we_selector_document',document.we_form.elements." . $prefix . "UrlID.value,WE().consts.tables.FILE_TABLE,'" . $prefix . "UrlID','" . $prefix . "UrlIDPath','setHot','',0,WE().consts.contentTypes.WEDOCUMENT," . (we_base_permission::hasPerm(we_base_perm::WORK_WORKSPACE_CAN_SELECT_OTHER_FILES) ? 0 : 1) . ")";

		$path = id_to_path($this->Model->UrlID);

		$weSuggest = & we_gui_suggest::getInstance();
		$weSuggest->setAcId($prefix . 'UrlIDPath');
		$weSuggest->setContentType(array_merge(we_base_ContentTypes::LINK_ABLE, [we_base_ContentTypes::FOLDER]));
		$weSuggest->setInput($prefix . 'UrlIDPath', $path, [], false, true);
		$weSuggest->setMaxResults(50);
		$weSuggest->setResult($prefix . 'UrlID', $this->Model->UrlID);
		$weSuggest->setSelector(we_gui_suggest::DocSelector);
		$weSuggest->setWidth(400);
		$weSuggest->setSelectButton(we_html_button::create_button(we_html_button::SELECT, $cmd));

		$weAcSelector = $weSuggest->getHTML();
		$selectionType = $this->Model->IsFolder ? $this->Model->DynamicSelection : $this->Model->SelectionType;

		return we_html_element::div(['id' => $prefix . 'LinkSelectionDiv', 'style' => 'display: ' . (($selectionType === we_navigation_navigation::STYPE_CATLINK || $selectionType == we_navigation_navigation::DYN_CATEGORY) ? 'block' : 'none') . ';margin-top: 5px;'], we_html_tools::formElementTable(g_l('navigation', '[linkSelection]'), we_html_tools::select($prefix . 'LinkSelection', [
						we_navigation_navigation::LSELECTION_INTERN => g_l('navigation', '[intern]'),
						we_navigation_navigation::LSELECTION_EXTERN => g_l('navigation', '[extern]')
						], 1, $this->Model->LinkSelection, false, ['style' => 'width: 520px;', 'onchange' => "setLinkSelection('" . $prefix . "',this.value); we_cmd('setHot');"]))) .
			we_html_element::div(['id' => $prefix . 'intern', 'style' => 'display: ' . (($this->Model->LinkSelection === we_navigation_navigation::LSELECTION_INTERN && $selectionType !== we_navigation_navigation::STYPE_URLLINK) ? 'block' : 'none') . ';margin-top: 5px;'], $weAcSelector) .
			we_html_element::div(['id' => $prefix . 'extern', 'style' => 'display: ' . (($this->Model->LinkSelection === we_navigation_navigation::LSELECTION_EXTERN || $selectionType === we_navigation_navigation::STYPE_URLLINK) ? 'block' : 'none') . ';margin-top: 5px;'], we_html_tools::textInput($prefix . 'Url', $this->Model->Url, '', [
					'onchange' => "we_cmd('setHot');", 'placeholder' => 'http://www.example.com'], 'text', 520)) .
			we_html_element::div(['style' => "margin-top: 5px;"], we_html_tools::formElementTable(g_l('navigation', '[catParameter]'), we_html_tools::textInput($prefix . 'CatParameter', $this->Model->CatParameter, '', [
						'onchange' => "we_cmd('setHot');"], 'text', 520)));
	}

	private function getHTMLCharsetTable(){
		$value = (!empty($this->Model->Charset) ? $this->Model->Charset : $GLOBALS['we']['Charset']['Backend']);

		$charsets = we_base_charsetHandler::inst()->getCharsetsForTagWizzard();
		asort($charsets);
		reset($charsets);

		$table = new we_html_table(['class' => 'default'], 1, 2);
		$table->setColContent(0, 0, we_html_tools::textInput("Charset", $value, '', [], 'text', 120));
		$table->setColContent(0, 1, we_html_tools::select("CharsetSelect", $charsets, 1, $value, false, ['onblur' => 'document.forms[0].elements.Charset.value=this.options[this.selectedIndex].value;',
				'onchange' => 'document.forms[0].elements.Charset.value=this.options[this.selectedIndex].value;'], true, 400, "defaultfont", false));

		return $table->getHtml();
	}

	private function getLangField($name, $value, $title, $width){
		//FIXME: these values should be obtained from global settings
		$input = we_html_tools::textInput("Attributes[$name]", $value, "", ['onchange' => "we_cmd('setHot');"], "text", $width - 100);
		$select = '
<select style="width:100px;" class="weSelect" name="' . $name . '_select" onchange="we_cmd(\'setHot\'); this.form.elements[\'Attributes[' . $name . ']\'].value=this.options[this.selectedIndex].value;this.selectedIndex=-1;">
	<option value=""></option>
	<option value="en">en</option>
	<option value="de">de</option>
	<option value="es">es</option>
	<option value="fi">fi</option>
	<option value="ru">ru</option>
	<option value="fr">fr</option>
	<option value="nl">nl</option>
	<option value="pl">pl</option>
</select>';
		return we_html_tools::formElementTable($title, $input, $select);
	}

	private function getRevRelSelect($type, $value, $title){
		$input = we_html_tools::textInput(
				'Attributes[' . $type . ']', $value, '', ['onchange' => "we_cmd('setHot');"], 'text', 400);
		$select = '<select name="' . $type . '_sel" class="weSelect" style="width:100px;" onchange="we_cmd(\'setHot\'); this.form.elements[\'Attributes[' . $type . ']\'].value=this.form.elements[\'Attributes[' . $type . ']\'].value+\' \'+this.options[this.selectedIndex].text;this.selectedIndex=0;">' . we_dialog_hyperlink::REV_REL_OPTIONS . '</select>';
		return we_html_tools::formElementTable($title, $input, $select);
	}

	private function getHTMLAttributes(){
		$title = we_html_tools::formElementTable(g_l('navigation', '[title]'), we_html_tools::textInput('Attributes[title]', $this->Model->getAttribute('title'), '', [
					'onchange' => "we_cmd('setHot');"], 'text', 520));

		$anchor = we_html_tools::formElementTable(g_l('navigation', '[anchor]'), we_html_tools::textInput('Attributes[anchor]', $this->Model->getAttribute('anchor'), '', [
					'onchange' => "we_cmd('setHot');", 'pattern' => '^#?[a-zA-Z]+[a-zA-Z0-9_:.-=]*$',
					'onblur' => "if(this.value&&!new RegExp(\'#?[a-z]+[a-z0-9_:.-=]*$\',\'i\').test(this.value)){WE().util.showMessage('" . g_l('linklistEdit', '[anchor_invalid]') . "', WE().consts.message.WE_MESSAGE_ERROR);}"], 'text', 520) . '<br/>' .
				we_html_forms::checkboxWithHidden($this->Model->CurrentOnAnker, 'CurrentOnAnker', g_l('navigation', '[current_on_anker]'), false, "defaultfont", "we_cmd('setHot');"));

		$target = we_html_tools::formElementTable(g_l('navigation', '[target]'), we_html_tools::targetBox('Attributes[target]', 400, '', $this->Model->getAttribute('target'), "we_cmd('setHot');", 100));

		$link = we_html_tools::formElementTable(g_l('navigation', '[link_attribute]'), we_html_tools::textInput('Attributes[link_attribute]', $this->Model->getAttribute('link_attribute'), '', [
					'onchange' => "we_cmd('setHot');"], 'text', 520));

		$lang = $this->getLangField('lang', $this->Model->getAttribute('lang'), g_l('navigation', '[link_language]'), 520);
		$hreflang = $this->getLangField('hreflang', $this->Model->getAttribute('hreflang'), g_l('navigation', '[href_language]'), 520);

		$parts = [
			[
				'headline' => '',
				'html' => we_html_tools::alertAttentionBox(g_l('navigation', '[linkprops_desc]'), we_html_tools::TYPE_INFO, 520),
				'space' => we_html_multiIconBox::SPACE_ICON,
				'noline' => true
			], [
				'headline' => g_l('navigation', '[attributes]'),
				'html' => $title . $anchor . $link . $target,
				'icon' => we_html_multiIconBox::PROP_ATTRIB,
				'noline' => true,
			], [
				'headline' => g_l('navigation', '[language]'),
				'html' => $lang . $hreflang,
				'icon' => we_html_multiIconBox::PROP_LANG,
				'noline' => true,
			]
		];

		$accesskey = we_html_tools::formElementTable(g_l('navigation', '[accesskey]'), we_html_tools::textInput('Attributes[accesskey]', $this->Model->getAttribute('accesskey'), '', [
					'onchange' => "we_cmd('setHot');"], 'text', 520));

		$tabindex = we_html_tools::formElementTable(g_l('navigation', '[tabindex]'), we_html_tools::textInput('Attributes[tabindex]', $this->Model->getAttribute('tabindex'), '', [
					'onchange' => "we_cmd('setHot');"], 'text', 520));

		$parts[] = [
			'headline' => g_l('navigation', '[keyboard]'),
			'html' => $accesskey . $tabindex,
			'icon' => we_html_multiIconBox::PROP_KEYBOARD,
			'noline' => true,
		];

		$relfield = $this->getRevRelSelect('rel', $this->Model->getAttribute('rel'), 'rel');

		$parts[] = [
			'headline' => g_l('navigation', '[relation]'),
			'html' => $relfield,
			'icon' => we_html_multiIconBox::PROP_REVREL,
			'noline' => true,
		];

		/* $input_width = 70;

		  $popup = new we_html_table(['class' => 'withSpace'], 4, 4);

		  $popup->setCol(0, 0, ['colspan' => 2], we_html_forms::checkboxWithHidden($this->Model->getAttribute('popup_open'), 'Attributes[popup_open]', g_l('navigation', '[popup_open]'), false, "defaultfont", 'top.content.mark();'));
		  $popup->setCol(0, 2, ['colspan' => 2], we_html_forms::checkboxWithHidden($this->Model->getAttribute('popup_center'), 'Attributes[popup_center]', g_l('navigation', '[popup_center]'), false, "defaultfont", 'top.content.mark();'));

		  $popup->setColContent(1, 0, we_html_tools::htmlFormElementTable(we_html_tools::textInput('Attributes[popup_xposition]', $this->Model->getAttribute('popup_xposition'), '', 'onchange="top.content.mark();', 'text', $input_width), g_l('navigation', '[popup_x]')));
		  $popup->setColContent(1, 1, we_html_tools::htmlFormElementTable(we_html_tools::textInput('Attributes[popup_yposition]', $this->Model->getAttribute('popup_yposition'), '', 'onchange="top.content.mark();', 'text', $input_width), g_l('navigation', '[popup_y]')));
		  $popup->setColContent(1, 2, we_html_tools::htmlFormElementTable(we_html_tools::textInput('Attributes[popup_width]', $this->Model->getAttribute('popup_width'), '', 'onchange="top.content.mark();', 'text', $input_width), g_l('navigation', '[popup_width]')));

		  $popup->setColContent(1, 3, we_html_tools::htmlFormElementTable(we_html_tools::textInput('Attributes[popup_height]', $this->Model->getAttribute('popup_height'), '', 'onchange="top.content.mark();', 'text', $input_width), g_l('navigation', '[popup_height]')));

		  $popup->setColContent(2, 0, we_html_forms::checkboxWithHidden($this->Model->getAttribute('popup_status'), 'Attributes[popup_status]', g_l('navigation', '[popup_status]'), false, "defaultfont", 'top.content.mark();'));
		  $popup->setColContent(2, 1, we_html_forms::checkboxWithHidden($this->Model->getAttribute('popup_scrollbars'), 'Attributes[popup_scrollbars]', g_l('navigation', '[popup_scrollbars]'), false, "defaultfont", 'top.content.mark();'));
		  $popup->setColContent(2, 2, we_html_forms::checkboxWithHidden($this->Model->getAttribute('popup_menubar'), 'Attributes[popup_menubar]', g_l('navigation', '[popup_menubar]'), false, "defaultfont", 'top.content.mark();'));

		  $popup->setColContent(3, 0, we_html_forms::checkboxWithHidden($this->Model->getAttribute('popup_resizable'), 'Attributes[popup_resizable]', g_l('navigation', '[popup_resizable]'), false, "defaultfont", 'top.content.mark();'));
		  $popup->setColContent(3, 1, we_html_forms::checkboxWithHidden($this->Model->getAttribute('popup_location'), 'Attributes[popup_location]', g_l('navigation', '[popup_location]'), false, "defaultfont", 'top.content.mark();'));
		  $popup->setColContent(3, 2, we_html_forms::checkboxWithHidden($this->Model->getAttribute('popup_toolbar'), 'Attributes[popup_toolbar]', g_l('navigation', '[popup_toolbar]'), false, "defaultfont", 'top.content.mark();'));

		  $parts[] = [
		  'headline' => g_l('navigation', '[popup]'),
		  'html' => $popup->getHTML(),
		  'icon' => we_html_multiIconBox::PROP_WINDOW,
		  'noline' => true,
		  'fold' => true
		  ]; */

		return $parts;
	}

	private function getHTMLImageAttributes(){
		$input_width = 70;
		$img_props = new we_html_table(['class' => 'withSpace'], 4, 5);

		$img_props->setColContent(0, 0, we_html_tools::formElementTable(g_l('navigation', '[icon_width]'), we_html_tools::textInput('Attributes[icon_width]', $this->Model->getAttribute('icon_width'), '', [
					'onchange' => "we_cmd('setHot');"], 'text', $input_width)));
		$img_props->setColContent(0, 1, we_html_tools::formElementTable(g_l('navigation', '[icon_height]'), we_html_tools::textInput('Attributes[icon_height]', $this->Model->getAttribute('icon_height'), '', [
					'onchange' => "we_cmd('setHot');"], 'text', $input_width)));
		$img_props->setColContent(0, 2, we_html_tools::formElementTable(g_l('navigation', '[icon_border]'), we_html_tools::textInput('Attributes[icon_border]', $this->Model->getAttribute('icon_border'), '', [
					'onchange' => "we_cmd('setHot');"], 'text', $input_width)));
		$img_props->setColContent(0, 3, we_html_tools::formElementTable(g_l('navigation', '[icon_hspace]'), we_html_tools::textInput('Attributes[icon_hspace]', $this->Model->getAttribute('icon_hspace'), '', [
					'onchange' => "we_cmd('setHot');"], 'text', $input_width)));
		$img_props->setColContent(0, 4, we_html_tools::formElementTable(g_l('navigation', '[icon_vspace]'), we_html_tools::textInput('Attributes[icon_vspace]', $this->Model->getAttribute('icon_vspace'), '', [
					'onchange' => "we_cmd('setHot');"], 'text', $input_width)));
		$img_props->setCol(1, 0, ['colspan' => 5], we_html_tools::formElementTable(g_l('navigation', '[icon_align]'), we_html_tools::select(
					'Attributes[icon_align]', [
					'' => 'Default',
					'top' => 'Top',
					'middle' => 'Middle',
					'bottom' => 'Bottom',
					'left' => 'left',
					'right' => 'Right',
					'texttop' => 'Text Top',
					'absmiddle' => 'Abd Middle',
					'baseline' => 'Baseline',
					'absbottom' => 'Abs Bottom'
					], 1, $this->Model->getAttribute('icon_align'), false, ['style' => 'width: 470px;'])));
		$img_props->setCol(2, 0, ['colspan' => 5], we_html_tools::formElementTable(g_l('navigation', '[icon_alt]'), we_html_tools::textInput('Attributes[icon_alt]', $this->Model->getAttribute('icon_alt'), '', [
					'onchange' => "we_cmd('setHot');"], 'text', 470)));
		$img_props->setCol(3, 0, ['colspan' => 5], we_html_tools::formElementTable(g_l('navigation', '[icon_title]'), we_html_tools::textInput('Attributes[icon_title]', $this->Model->getAttribute('icon_title'), '', [
					'onchange' => "we_cmd('setHot');"], 'text', 470)));

		return $img_props->getHTML();
	}

	private function getHTMLTab3(){
		$filter = new we_navigation_customerFilter();
		$filter->initByNavModel($this->Model);

		$view = new we_navigation_customerFilterView($filter, 520);
		return [
			[
				'headline' => '',
				'html' => $view->getFilterHTML($this->Model->IsFolder == 0 && $this->Model->Selection == we_navigation_navigation::SELECTION_DYNAMIC),
				'space' => we_html_multiIconBox::SPACE_MED,
				'noline' => true
		]];
	}

	protected function getHTMLEditorFooter(array $btn_cmd = [], $extraHead = ''){
		if(we_base_request::_(we_base_request::BOOL, 'home')){
			return parent::getHTMLEditorFooter([]);
		}

		$buttons = we_html_element::div(['class' => 'main'],
																	we_html_button::create_button(we_html_button::FOOTER_SAVE, "javascript:top.content.makeNewDoc=document.we_form.makeNewDoc.checked;top.content.we_cmd('module_navigation_save');", '', '', (!we_base_permission::hasPerm(we_base_perm::NAVIGATION_EDIT))) .
				we_html_forms::checkbox('makeNewDoc', false, 'makeNewDoc', g_l('global', ($this->View->Model->IsFolder ? '[we_new_folder_after_save]' : '[we_new_entry_after_save]')))
		);
		if($this->Model->ID && we_base_permission::hasPerm([we_base_perm::NAVIGATION_DELETE, we_base_perm::NAVIGATION_EDIT])){
			$buttons .= we_html_element::div(['class' => 'optional'],
																		we_html_button::create_button(we_html_button::FOOTER_TRASH, "javascript:top.content.we_cmd('module_navigation_delete');"));
		}

		return $this->getHTMLDocument(
				we_html_element::body([
					'id' => 'footerBody',
					'onload' => 'document.we_form.makeNewDoc.checked=top.content.makeNewDoc;'
					], we_html_element::form([], we_html_element::div(['class' => 'footertable'], $buttons))));
	}

}
