<?php
include_once(WE_INCLUDES_PATH . 'we_language/' . $GLOBALS['WE_LANGUAGE'] . '/taged.inc.php');

/**
 * webEdition CMS
 *
 * $Rev: 3b2f6882793a $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2018-08-27 16:57 +0000 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_tagData_attribute{
	/**
	 * @var string
	 */
	protected $Id;

	/**
	 * @var string
	 */
	protected $Name;

	/**
	 * @var boolean
	 */
	protected $Required;

	/**
	 * @var string
	 */
	protected $Module;

	/**
	 * @var string
	 */
	protected $Value;
	protected $Description;
	protected $Deprecated;

	/**
	 * @param string $name
	 * @param boolean $required
	 * @param string $module
	 */
	public function __construct($name, $required = false, $module = '', $description = '', $deprecated = false){
		static $count = 0;
		$this->Id = ++$count;
		$this->Name = $name;
		$this->Required = $required;
		$this->Module = $module;
		// set value occasionally
		$this->Value = we_base_request::_(we_base_request::STRING, 'attributes', false, $name);
		$this->Description = $description;
		$this->Deprecated = $deprecated;
	}

	/**
	 * @return string
	 */
	protected function getLabelCodeForTagWizard(){
		$tmp = [
			'id' => 'label_' . $this->getIdName(),
			'class' => 'defaultfont',
			'for' => $this->getIdName()
		];
		if($this->Description){
			$tmp['style'] = 'border-bottom-style: dotted;border-bottom-width: 1px;border-spacing: 2px;cursor:help;';
			$tmp['title'] = $this->Description;
		}
		if($this->Deprecated){
			$tmp['style'] = (empty($tmp['style']) ? '' : $tmp['style']) . 'text-decoration:line-through;';
		}
		return we_html_element::label($tmp, $this->Name . ($this->Required ? '*' : ''));
	}

	/**
	 * @return string
	 */
	public function getName(){
		return $this->Name;
	}

	/**
	 * @return string
	 */
	public function getIdName(){
		return 'id' . $this->Id . '_' . $this->Name;
	}

	/**
	 * @return boolean
	 */
	public function IsRequired(){
		return $this->Required;
	}

	public function IsDeprecated(){
		return $this->Deprecated;
	}

	public function getDescription(){
		return $this->Description;
	}

	/**
	 * checks if this attribute should be used, checks if needed modules are installed
	 * @return boolean
	 */
	public function useAttribute(){
		return ($this->Module === '' || we_modules_info::isActive($this->Module));
	}

	/**
	 * checks if this option should be used, checks if needed modules are installed
	 * @return array
	 */
	public static function getUseOptions(array $options){
		$useOptions = [];
		foreach($options as $option){
			if($option->useOption()){
				$useOptions[] = $option;
			}
		}
		return $useOptions;
	}

}

//FIXME: remove
class weTagDataAttribute extends we_tagData_attribute{

	public function __construct($name, $required = false, $module = '', $description = '', $deprecated = false){
		t_e('deprecated', 'this class is removed in 8.1');
		parent::__construct($name, $required, $module, $description, $deprecated);
	}

}
