<?php
/**
 * webEdition CMS
 *
 * $Rev: 342e215a2edb $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2018-03-21 11:07 +0000 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * Baseclass for all UI relevant functions
 * all functions not found in extended classes will be redirected to their corresponding document which holds the data
 */
abstract class we_view_base{
	protected $doc;

	public function __call($name, $params){
		t_e('call, use $this->doc->', $name);
		return call_user_func_array([$this->doc, $name], $params);
	}

	public function __get($name){
		t_e('get, use $this->doc->', $name);
		return $this->doc->$name;
	}

	public function __set($name, $value){
		t_e('set, use $this->doc->', $name);
		$this->doc->$name = $value;
	}

	public function __isset($name){
		t_e('isset, use $this->doc->', $name);
		return isset($this->doc->$name);
	}

	public static function getView(we_contents_root $doc){
		switch(($class = get_class($doc))){
			case 'we_backup_object':
			case 'we_doctype_docTypes'://fixme: should this have a view class?
				return null;
			default:
				$cnt = 0;
				$nClass = str_replace(['_contents_', '_document_'], '_view_', $class, $cnt);
				if(!$cnt){
					$nClass = str_replace('we_', 'we_view_', $class, $cnt);
				}
				if($cnt && class_exists($nClass)){
					return new $nClass($doc);
				}
				t_e('no view for doc found', $class);
				return null;
		}
	}

	protected function __construct(we_contents_base $doc){
		$this->doc = $doc;
	}

}
