<?php
/**
 * webEdition CMS
 *
 * $Rev: 52ac818a7d19 $
 * $Author: Marc Krämer <marc.kraemer@aldente-it.de> $
 * $Date: 2018-12-20 01:05 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
we_include_tag_file('we_tag_blockControls');

abstract class we_tag_block extends we_weTag_tag{
	static $blkCnt = -1;

	public static function parse(array $attribs, $content){
		if(($foo = self::attributeMissing($attribs, 'name', __CLASS__))){
			return $foo;
		}

		//cleanup content
		while(strpos($content, '\\\\$') !== false){
			$content = str_replace('\\\\$', '$', $content);
		}
		//replace all \$ which are not inside we-tags
		$content = preg_replace('|([^\'"])\\\\\$|', '${1}\$', $content);

		$blockName = self::getParserAttribute('name', $attribs);
		$name = str_replace(['$', '.', '/', 0, 1, 2, 3, 4, 5, 6, 7, 8, 9], '', md5($blockName)) . ( ++self::$blkCnt);

		$count = 0;
		if(($content = str_replace('we_tag_block::controls("##blockControlsREPL##"', 'we_tag_block::controls($block_' . $name, $content, $count)) && $count){
			//nothing to do, we have a userdefined blockcontrol
		} else {
			$content = (preg_match('/< ?(tr|td)/i', $content) ?
				//table found
				strtr(preg_replace('|(< ?td[^>]*>)|si', '${1}' . '<?php we_tag_block::controls($block_' . $name . ');?>', strtr($content, [
					'=>' => '#####PHPC####',
					'?>' => '#####PHPE####']), 1), [
					'#####PHPC####' => '=>',
					'#####PHPE####' => '?>']) :
				//no tables found
				'<?php we_tag_block::controls($block_' . $name . ');?>' . $content
				);
		}
//	}
		//here postTagName is explicitly needed, because the control-element is not "inside" the block-tag (no block defined/first element) but controls its elements
		return '<?php
$block_' . $name . '=' . we_weTag_parser::printTag('block', $attribs) . ';
while(we_tag_block::condition($block_' . $name . ')){
	?>' . $content . '<?php
}
unset($block_' . $name . '); ?>';
	}

	public static function tag(array $attribs = [], $content = '', $internal = false){
		$origName = self::getAttribute('_name_orig', $attribs, '', we_base_request::STRING);
		$name = self::getAttribute('name', $attribs, '', we_base_request::STRING);
		$start = self::getAttribute('start', $attribs, 0, we_base_request::INT);
		$limit = self::getAttribute('limit', $attribs, 0, we_base_request::INT);

		if(we_tag_listview::$current){
			if(!($list = we_tag_listview::$current->f($name))){
				$list = we_tag_listview::$current->f($origName);
				$name = $origName;
			}
		} else {
			$list = $GLOBALS['we_doc']->getElement($name);
		}

		if(($list = we_unserialize($list, [], true))){
			if(is_array($list) && count($list) && ((count($list) - 1) != max(array_keys($list)))){
				//reorder list!
				$list = array_values($list);
				$GLOBALS['we_doc']->setElement($name, we_serialize($list, SERIALIZE_JSON, true, 0, true));
			}
		} else if($start){
			$list = [];
			if($limit > 0){
				$start = min($start, $limit);
			}
			for($i = 1; $i <= $start; $i++){
				$list[] = '_' . $i;
			}
		}

		$listlen = count($list);
		$show = 10;
		if($limit > 0){
			if($GLOBALS['we']['editmode']){
				$diff = $limit - $listlen;
				$show = ($diff > 0 ? min($show, $diff) : 0);
			} else {
				$listlen = min($listlen, $limit);
			}
		}
		return [
			'name' => $name,
			'list' => $list,
			'listSize' => $listlen,
			'ctlShow' => $show,
			'ctlShowSelect' => self::getAttribute('showselect', $attribs, true, we_base_request::BOOL),
			'pos' => -1,
			'lastPostName' => isset($GLOBALS['we']['postTagName']) ? $GLOBALS['we']['postTagName'] : '',
		];
	}

	public static function condition(&$block){
		if(!is_array($block)){
			return false;
		}
		//go to next element
		++$block['pos'];
		if($block['pos'] >= $block['listSize']){
			//end of list
			//we need a last add button in editmode
			if($GLOBALS['we']['editmode']){
				self::controls($block);
			}
			//reset data
			unset($GLOBALS['we_position']['block'][$block['name']]);
			$GLOBALS['we']['postTagName'] = $block['lastPostName'];
			return false;
		}

		$GLOBALS['we']['postTagName'] = 'blk_' . $block['name'] . '_' . $block['list'][$block['pos']];

		$GLOBALS['we_position']['block'][$block['name']] = [
			'position' => $block['pos'] + 1,
			'size' => $block['listSize']
		];
		return true;
	}

	public static function controls(array $attribs){
		//if in listview no Buttons are shown!
		if(!$GLOBALS['we']['editmode'] || we_tag_listview::$current || !isset($attribs['list']) || !self::getAttribute('show', $attribs, true, we_base_request::BOOL)){
			return '';
		}
		$showSelect = self::getAttribute('showselect', $attribs, $attribs['ctlShowSelect'], we_base_request::BOOL);

		if(!isset($attribs['ctlName'])){
			$attribs['ctlName'] = md5(str_replace('.', '', uniqid('', true)));
		}

		if($attribs['pos'] < $attribs['listSize']){
			$tabArray = [];
			if($showSelect && $attribs['ctlShow'] > 0){
				$jsSelector = $attribs['pos'] . ",document.we_form.elements['" . $attribs['ctlName'] . '_' . $attribs['pos'] . "'].options[document.we_form.elements['" . $attribs['ctlName'] . '_' . $attribs['pos'] . "'].selectedIndex].text";
				$tabArray[] = we_html_button::create_button(we_html_button::LIST_ADD, "javascript:setScrollTo();_EditorFrame.setEditorIsHot(true);we_cmd('insert_entry_at_list','" . $attribs['name'] . "'," . $jsSelector . ")", '', '', !($attribs['ctlShow'] > 0));

				$selectb = '<select name="' . $attribs['ctlName'] . '_' . $attribs['pos'] . '">';
				for($j = 0; $j < $attribs['ctlShow']; $j++){
					$selectb .= '<option value="' . ($j + 1) . '">' . ($j + 1) . '</option>';
				}
				$selectb .= '</select>';
				$tabArray[] = $selectb;
			} else {
				$tabArray[] = we_html_button::create_button(we_html_button::LIST_ADD, "javascript:setScrollTo();_EditorFrame.setEditorIsHot(true);we_cmd('insert_entry_at_list','" . $attribs['name'] . "','" . $attribs['pos'] . "',1)", '', '', !($attribs['ctlShow'] > 0));
				$jsSelector = 1;
			}
			$tabArray[] = (($attribs['pos'] > 0) ?
				//enabled upBtn
				we_html_button::create_button(we_html_button::DIRUP, "javascript:setScrollTo();_EditorFrame.setEditorIsHot(true);we_cmd('up_entry_at_list','" . $attribs['name'] . "','" . $attribs['pos'] . "'," . $jsSelector . ")") :
				//disabled upBtn
				we_html_button::create_button(we_html_button::DIRUP, '', '', '', true));
			$tabArray[] = (($attribs['pos'] == $attribs['listSize'] - 1) ?
				//disabled downBtn
				we_html_button::create_button(we_html_button::DIRDOWN, '', '', '', true) :
				//enabled downBtn
				we_html_button::create_button(we_html_button::DIRDOWN, "javascript:setScrollTo();_EditorFrame.setEditorIsHot(true);we_cmd('down_entry_at_list','" . $attribs['name'] . "','" . $attribs['pos'] . "'," . $jsSelector . ")"));
			$tabArray[] = we_html_button::create_button(we_html_button::TRASH, "javascript:setScrollTo();_EditorFrame.setEditorIsHot(true);we_cmd('delete_list','" . $attribs['name'] . "','" . $attribs['pos'] . "','" . $GLOBALS['we']['postTagName'] . "')");

			echo '<div class="we we_blockControls">' . implode('', $tabArray) . '</div>';
			return '';
		}
		if($showSelect && $attribs['ctlShow'] > 0){
			$selectb = '<select name="' . $attribs['ctlName'] . '_00">';
			for($j = 1; $j <= $attribs['ctlShow']; $j++){
				$selectb .= '<option value="' . $j . '">' . $j . '</option>';
			}
			$selectb .= '</select>';
			$plusbut = '<div class="we we_blockControls">' .
				we_html_button::create_button(we_html_button::LIST_ADD, "javascript:setScrollTo();_EditorFrame.setEditorIsHot(true);we_cmd('add_entry_to_list','" . $attribs['name'] . "',document.we_form.elements['" . $attribs['ctlName'] . "_00'].options[document.we_form.elements['" . $attribs['ctlName'] . "_00'].selectedIndex].text);", '', '', !($attribs['ctlShow'] > 0)) .
				$selectb .
				'</div>';
		} else {
			$plusbut = '<div class="we we_blockControls">' .
				we_html_button::create_button(we_html_button::LIST_ADD, "javascript:setScrollTo();_EditorFrame.setEditorIsHot(true);we_cmd('add_entry_to_list','" . $attribs['name'] . "',1)", '', '', !($attribs['ctlShow'] > 0)) .
				'</div>';
		}

		echo '<input type="hidden" name="we_' . $GLOBALS['we_doc']->Name . '_block[' . $attribs['name'] . ']" value="' .
		htmlentities(
			(empty($attribs['list']) ? '' : we_serialize($attribs['list'], SERIALIZE_JSON, true, 0, true))
		) . //FIXME: do we really need this serialized in the document???
		'">' . $plusbut;
		return '';
	}

}

function we_condition_tag_block(&$block){ //FIXME:remove
	return we_tag_block::condition($block);
}

function we_tag_block(array $attribs){
	return we_tag_block::tag($attribs);
}
